@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2004 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,25 +7,23 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifndef	BZF_BUNDLEMGR_H
 #define	BZF_BUNDLEMGR_H
 
-#ifdef _MSC_VER
-#pragma warning(4:4786)
-#endif
-
-#include <string>
-#include <vector>
-#include <map>
 #include "common.h"
 
+/* system interface headers */
+#include <map>
+#include <string>
+#include <vector>
 #ifdef __APPLE__
-#import <CoreFoundation/CoreFoundation.h>
+#  include <CoreFoundation/CoreFoundation.h>
 #endif
 
+
 class Bundle;
 
 typedef std::map<std::string,Bundle *> BundleMap;
