@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 #ifndef _TEXTURE_MANAGER_H
 #define _TEXTURE_MANAGER_H
@@ -51,12 +51,15 @@ class TextureManager : public Singleton<TextureManager>
 {
 public:
   int getTextureID( const char* name, bool reportFail = true );
-  int addTexture( const char*, OpenGLTexture *texture  );
 
-  void setTextureFilter(int texId, OpenGLTexture::Filter filter);
-  OpenGLTexture::Filter getTextureFilter(int texId);
+  bool isLoaded(const std::string& name);
+  bool removeTexture(const std::string& name);
+  bool reloadTextures();
+  bool reloadTextureImage(const std::string& name);
 
   void updateTextureFilters();
+  void setTextureFilter(int texId, OpenGLTexture::Filter filter);
+  OpenGLTexture::Filter getTextureFilter(int texId);
 
   bool bind ( int id );
   bool bind ( const char* name );
@@ -64,6 +67,8 @@ public:
   const ImageInfo& getInfo ( int id );
   const ImageInfo& getInfo ( const char* name );
 
+  bool getColorAverages(int texId, float rgba[4], bool factorAlpha) const;
+
   OpenGLTexture::Filter getMaxFilter ( void );
   std::string getMaxFilterName ( void );
   void setMaxFilter ( OpenGLTexture::Filter filter );
@@ -82,6 +87,7 @@ private:
   TextureManager& operator=(const TextureManager &tm);
   ~TextureManager();
 
+  int addTexture( const char*, OpenGLTexture *texture  );
   OpenGLTexture* loadTexture( FileTextureInit &init, bool reportFail = true  );
 
   typedef std::map<std::string, ImageInfo> TextureNameMap;
@@ -93,6 +99,7 @@ private:
   TextureNameMap textureNames;
 };
 
+
 #endif //_TEXTURE_MANAGER_H
 
 // Local Variables: ***
