@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,16 +7,21 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#include "TimeKeeper.h"
+// get our interface
+#include "AutoPilot.h"
+
+/* common headers */
+#include "BZDBCache.h"
+#include "BoxBuilding.h"
+
+/* local headers */
 #include "Roster.h"
 #include "TargetingUtils.h"
 #include "World.h"
 #include "WorldPlayer.h"
-#include "BZDBCache.h"
-#include "ServerLink.h"
 #include "playing.h"
 #include "Plan.h"
 
@@ -46,7 +51,7 @@ void teachAutoPilot(FlagType *type, int adjust)
   totalCnt++;
 }
 
-bool isFlagUseful(FlagType *type)
+static bool isFlagUseful(FlagType *type)
 {
   if (type == Flags::Null)
     return false;
@@ -71,7 +76,7 @@ bool isFlagUseful(FlagType *type)
   return ((float)flagValue) >= avg;
 }
 
-ShotPath *findWorstBullet(float &minDistance)
+static ShotPath *findWorstBullet(float &minDistance)
 {
   LocalPlayer *myTank = LocalPlayer::getMyTank();
   const float *pos = myTank->getPosition();
@@ -106,7 +111,7 @@ ShotPath *findWorstBullet(float &minDistance)
       if (dist < minDistance) {
 	const float *shotVel = shot->getVelocity();
 	float shotAngle = atan2f(shotVel[1], shotVel[0]);
-	float shotUnitVec[2] = {cos(shotAngle), sin(shotAngle)};
+	float shotUnitVec[2] = {cosf(shotAngle), sinf(shotAngle)};
 
 	float trueVec[2] = { (pos[0] - shotPos[0]) / dist, (pos[1] - shotPos[1]) / dist };
 	float dotProd = trueVec[0] * shotUnitVec[0] + trueVec[1] * shotUnitVec[1];
@@ -139,7 +144,7 @@ ShotPath *findWorstBullet(float &minDistance)
     if (dist < minDistance) {
       const float *shotVel = shot->getVelocity();
       float shotAngle = atan2f(shotVel[1], shotVel[0]);
-      float shotUnitVec[2] = {cos(shotAngle), sin(shotAngle)};
+      float shotUnitVec[2] = {cosf(shotAngle), sinf(shotAngle)};
 
       float trueVec[2] = { (pos[0] - shotPos[0]) / dist, (pos[1] - shotPos[1]) / dist };
       float dotProd = trueVec[0] * shotUnitVec[0] + trueVec[1] * shotUnitVec[1];
@@ -156,7 +161,7 @@ ShotPath *findWorstBullet(float &minDistance)
   return minPath;
 }
 
-bool	avoidDeathFall(float &/*rotation*/, float &speed)
+static bool avoidDeathFall(float &/*rotation*/, float &speed)
 {
   LocalPlayer *myTank = LocalPlayer::getMyTank();
   float pos1[3], pos2[3];
@@ -188,7 +193,7 @@ bool	avoidDeathFall(float &/*rotation*/, float &speed)
   return false;
 }
 
-bool	avoidBullet(float &rotation, float &speed)
+static bool avoidBullet(float &rotation, float &speed)
 {
   LocalPlayer *myTank = LocalPlayer::getMyTank();
   const float *pos = myTank->getPosition();
@@ -205,22 +210,15 @@ bool	avoidBullet(float &rotation, float &speed)
   const float *shotPos = shot->getPosition();
   const float *shotVel = shot->getVelocity();
   float shotAngle = atan2f(shotVel[1],shotVel[0]);
-  float shotUnitVec[2] = {cos(shotAngle), sin(shotAngle)};
+  float shotUnitVec[2] = {cosf(shotAngle), sinf(shotAngle)};
 
   float trueVec[2] = {(pos[0]-shotPos[0])/minDistance,(pos[1]-shotPos[1])/minDistance};
   float dotProd = trueVec[0]*shotUnitVec[0]+trueVec[1]*shotUnitVec[1];
 
-#ifdef _MSC_VER
   if (((World::getWorld()->allowJumping() || (myTank->getFlag()) == Flags::Jumping
    || (myTank->getFlag()) == Flags::Wings))
-   && (minDistance < (max(dotProd,0.5f) * BZDBCache::tankLength * 2.25f))
+   && (minDistance < ( std::max(dotProd, 0.5f) * BZDBCache::tankLength * 2.25f))
    && (myTank->getFlag() != Flags::NoJumping)) {
-#else
-  if (((World::getWorld()->allowJumping() || (myTank->getFlag()) == Flags::Jumping
-   || (myTank->getFlag()) == Flags::Wings))
-   && (minDistance < (std::max(dotProd,0.5f) * BZDBCache::tankLength * 2.25f))
-   && (myTank->getFlag() != Flags::NoJumping)) {
-#endif
     wantJump = true;
     return (myTank->getFlag() != Flags::Wings);
   } else if (dotProd > 0.96f) {
@@ -254,12 +252,12 @@ bool	avoidBullet(float &rotation, float &speed)
   return false;
 }
 
-bool	stuckOnWall(float &rotation, float &speed)
+static bool stuckOnWall(float &rotation, float &speed)
 {
   static TimeKeeper lastStuckTime;
   static float stuckRot = 0.0f, stuckSpeed = 0.0f;
 
-  float stuckPeriod = TimeKeeper::getTick() - lastStuckTime;
+  float stuckPeriod = float(TimeKeeper::getTick() - lastStuckTime);
   if (stuckPeriod < 0.5f) {
     rotation = stuckRot;
     speed = stuckSpeed;
@@ -299,7 +297,7 @@ bool	stuckOnWall(float &rotation, float &speed)
   return false;
 }
 
-RemotePlayer *findBestTarget()
+static RemotePlayer *findBestTarget()
 {
   RemotePlayer *target = NULL;
   LocalPlayer *myTank = LocalPlayer::getMyTank();
@@ -329,7 +327,7 @@ RemotePlayer *findBestTarget()
 	  (myTank->getTeam() == RedTeam && player[t]->getFlag() == Flags::RedTeam) ||
 	  (myTank->getTeam() == GreenTeam && player[t]->getFlag() == Flags::GreenTeam) ||
 	  (myTank->getTeam() == BlueTeam && player[t]->getFlag() == Flags::BlueTeam) ||
-				(myTank->getTeam() == PurpleTeam && player[t]->getFlag() == Flags::PurpleTeam)) {
+	  (myTank->getTeam() == PurpleTeam && player[t]->getFlag() == Flags::PurpleTeam)) {
 	target = player[t];
 	break;
       }
@@ -354,27 +352,27 @@ RemotePlayer *findBestTarget()
   return target;
 }
 
-bool chasePlayer(float &rotation, float &speed)
+static bool chasePlayer(float &rotation, float &speed)
 {
   LocalPlayer *myTank = LocalPlayer::getMyTank();
   const float *pos = myTank->getPosition();
 
-  RemotePlayer *player = findBestTarget();
-  if (player == NULL)
+  RemotePlayer *rPlayer = findBestTarget();
+  if (rPlayer == NULL)
     return false;
 
-  myTank->setTarget(player);
+  myTank->setTarget(rPlayer);
 
-  const float *targetPos = player->getPosition();
+  const float *targetPos = rPlayer->getPosition();
   float distance = TargetingUtils::getTargetDistance(pos, targetPos);
   if (distance > 250.0f)
     return false;
 
-  const float *tp = player->getPosition();
+  const float *tp = rPlayer->getPosition();
   float enemyPos[3];
   //toss in some lag adjustment/future prediction - 300 millis
   memcpy(enemyPos,tp,sizeof(enemyPos));
-  const float *tv = player->getVelocity();
+  const float *tv = rPlayer->getVelocity();
   enemyPos[0] += 0.3f * tv[0];
   enemyPos[1] += 0.3f * tv[1];
   enemyPos[2] += 0.3f * tv[2];
@@ -431,8 +429,8 @@ bool chasePlayer(float &rotation, float &speed)
   const Player *target = myTank->getTarget();
   if ((distance > (BZDB.eval(StateDatabase::BZDB_SHOTSPEED) /2.0f))
       ||  (myTank->getFiringStatus() != LocalPlayer::Ready)) {
-    float enemyUnitVec[2] = { cos(enemyAzimuth), sin(enemyAzimuth) };
-    float myUnitVec[2] = { cos(myAzimuth), sin(myAzimuth) };
+    float enemyUnitVec[2] = { cosf(enemyAzimuth), sinf(enemyAzimuth) };
+    float myUnitVec[2] = { cosf(myAzimuth), sinf(myAzimuth) };
     float dotProd = (myUnitVec[0] * enemyUnitVec[0] + myUnitVec[1] * enemyUnitVec[1]);
     if (dotProd < 0.866f) {
       //if target is more than 30 degrees away, turn as fast as you can
@@ -454,7 +452,7 @@ bool chasePlayer(float &rotation, float &speed)
   return true;
 }
 
-bool lookForFlag(float &rotation, float &speed)
+static bool lookForFlag(float &rotation, float &speed)
 {
   LocalPlayer *myTank = LocalPlayer::getMyTank();
   float pos[3];
@@ -477,17 +475,17 @@ bool lookForFlag(float &rotation, float &speed)
       continue;
 
     if (world->getFlag(i).type->flagTeam != NoTeam)
-		teamFlag = i;
+      teamFlag = i;
     const float* fpos = world->getFlag(i).position;
     if (fpos[2] == pos[2]) {
       float dist = TargetingUtils::getTargetDistance(pos, fpos);
       bool isTargetObscured = TargetingUtils::isLocationObscured(pos, fpos);
       if (isTargetObscured)
-		dist *= 1.25f;
+	dist *= 1.25f;
 
       if ((dist < 200.0f) && (dist < minDist)) {
-		minDist = dist;
-		closestFlag = i;
+	minDist = dist;
+	closestFlag = i;
       }
     }
   }
@@ -497,8 +495,8 @@ bool lookForFlag(float &rotation, float &speed)
   if (closestFlag != -1) {
     if (minDist < 10.0f) {
       if (myTank->getFlag() != Flags::Null) {
-		serverLink->sendDropFlag(myTank->getPosition());
-		handleFlagDropped(myTank);
+	serverLink->sendDropFlag(myTank->getPosition());
+	handleFlagDropped(myTank);
       }
     }
 
@@ -513,7 +511,7 @@ bool lookForFlag(float &rotation, float &speed)
   return false;
 }
 
-bool navigate(float &rotation, float &speed)
+static bool navigate(float &rotation, float &speed)
 {
   static TimeKeeper lastNavChange;
   static float navRot = 0.0f, navSpeed = 0.0f;
@@ -552,17 +550,18 @@ bool navigate(float &rotation, float &speed)
     if (temp == NULL) {
       serverLink->sendDropFlag(myTank->getPosition());
       handleFlagDropped(myTank);
-    }
-    if ((((int) *(world->getBase(myTank->getTeam())) + 2
-	  >= (int) *(myTank->getPosition()))
-	 || (temp[0] == pos[0] && temp[1] == pos[1])) &&
-	myTank->getFlag()->flagTeam == myTank->getTeam()) {
-      serverLink->sendDropFlag(myTank->getPosition());
-      handleFlagDropped(myTank);
     } else {
-      float baseAzimuth = TargetingUtils::getTargetAzimuth(pos, temp);
-      rotation = TargetingUtils::getTargetRotation(myAzimuth, baseAzimuth);
-      speed = (float)(M_PI/2.0 - fabs(rotation));
+      if ((((int) *(world->getBase(myTank->getTeam())) + 2
+	    >= (int) *(myTank->getPosition()))
+	   || (temp[0] == pos[0] && temp[1] == pos[1])) &&
+	  myTank->getFlag()->flagTeam == myTank->getTeam()) {
+	serverLink->sendDropFlag(myTank->getPosition());
+	handleFlagDropped(myTank);
+      } else {
+	float baseAzimuth = TargetingUtils::getTargetAzimuth(pos, temp);
+	rotation = TargetingUtils::getTargetRotation(myAzimuth, baseAzimuth);
+	speed = (float)(M_PI/2.0 - fabs(rotation));
+      }
     }
   } else {
     speed = 1.0f;
@@ -577,7 +576,7 @@ bool navigate(float &rotation, float &speed)
   return true;
 }
 
-bool fireAtTank()
+static bool fireAtTank()
 {
   static TimeKeeper lastShot;
   float pos[3];
@@ -642,7 +641,8 @@ bool fireAtTank()
 	    myTank->validTeamTarget(player[t])) {
 
 	  if (player[t]->isPhantomZoned() && !myTank->isPhantomZoned()
-	      && (myTank->getFlag() != Flags::SuperBullet))
+	      && (myTank->getFlag() != Flags::SuperBullet)
+	&& (myTank->getFlag() != Flags::ShockWave))
 	    continue;
 
 	  const float *tp = player[t]->getPosition();
@@ -685,7 +685,7 @@ bool fireAtTank()
   return false;
 }
 
-void    dropHardFlags()
+static void dropHardFlags()
 {
   LocalPlayer *myTank = LocalPlayer::getMyTank();
   FlagType *type = myTank->getFlag();
