@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,45 +7,36 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-// bzflag common headers
-#include "common.h"
-#include "global.h"
-
 // interface header
 #include "BackgroundRenderer.h"
 
 // system headers
-#include <stdio.h>
-#include <math.h>
-#include <string>
+#include <string.h>
 
 // common headers
-#include "bzfgl.h"
-#include "OpenGLGState.h"
 #include "OpenGLMaterial.h"
 #include "TextureManager.h"
-#include "StateDatabase.h"
 #include "BZDBCache.h"
 #include "BzMaterial.h"
+#include "TextureMatrix.h"
 #include "ParseColor.h"
+#include "BZDBCache.h"
 
 // local headers
 #include "daylight.h"
 #include "stars.h"
 #include "MainWindow.h"
 #include "SceneNode.h"
-#include "SceneRenderer.h"
-#include "WeatherRenderer.h"
-
-//static     bool	 useMoonTexture = false;
+#include "effectsRenderer.h"
 
 static const GLfloat	squareShape[4][2] =
 				{ {  1.0f,  1.0f }, { -1.0f,  1.0f },
 				  { -1.0f, -1.0f }, {  1.0f, -1.0f } };
 
+
 GLfloat			BackgroundRenderer::skyPyramid[5][3];
 const GLfloat		BackgroundRenderer::cloudRepeats = 3.0f;
 static const int	NumMountainFaces = 16;
@@ -115,6 +106,7 @@ BackgroundRenderer::BackgroundRenderer(const SceneRenderer&) :
   memcpy(moonDirection, up, sizeof(float[3]));
 
   // make ground materials
+  setupSkybox();
   setupGroundMaterials();
 
   TextureManager &tm = TextureManager::instance();
@@ -126,13 +118,10 @@ BackgroundRenderer::BackgroundRenderer(const SceneRenderer&) :
   gridGState = gstate.getState();
 
   // make receiver stuff
-  {
-    // gstates
-    gstate.reset();
-    gstate.setShading();
-    gstate.setBlending((GLenum)GL_ONE, (GLenum)GL_ONE);
-    receiverGState = gstate.getState();
-  }
+  gstate.reset();
+  gstate.setShading();
+  gstate.setBlending((GLenum)GL_SRC_ALPHA, (GLenum)GL_ONE);
+  receiverGState = gstate.getState();
 
   // sun shadow stuff
   gstate.reset();
@@ -183,7 +172,9 @@ BackgroundRenderer::BackgroundRenderer(const SceneRenderer&) :
   }
 
   // rain stuff
-	weather.init();
+  weather.init();
+  // effects
+  EFFECTS.init();
 
   // make mountain stuff
   mountainsAvailable = false;
@@ -287,8 +278,8 @@ void BackgroundRenderer::setupGroundMaterials()
   // see if we have a map specified material
   const BzMaterial* bzmat = MATERIALMGR.findMaterial("GroundMaterial");
 
-  int groundTextureID = -1;
-  int groundTextureMatrixID = -1;
+  groundTextureID = -1;
+  groundTextureMatrix = NULL;
 
   if (bzmat == NULL) {
     // default ground material
@@ -297,18 +288,23 @@ void BackgroundRenderer::setupGroundMaterials()
   }
   else {
     // map specified material
+    ((BzMaterial*)bzmat)->setReference();
     for (int i = 0; i < 4; i++) {
       memcpy (groundColor[i], bzmat->getDiffuse(), sizeof(GLfloat[4]));
     }
     if (bzmat->getTextureCount() > 0) {
-      groundTextureID = tm.getTextureID(bzmat->getTexture(0).c_str(), false);
+      groundTextureID = tm.getTextureID(bzmat->getTextureLocal(0).c_str(), false);
       if (groundTextureID < 0) {
 	// use the default as a backup (default color too)
 	memcpy (groundColor, defaultGroundColor, sizeof(GLfloat[4][4]));
 	groundTextureID = tm.getTextureID(BZDB.get("stdGroundTexture").c_str(), true);
       } else {
 	// only apply the texture matrix if the texture is valid
-	groundTextureMatrixID = bzmat->getTextureMatrix(0);
+	const int texMatId = bzmat->getTextureMatrix(0);
+	const TextureMatrix* texmat = TEXMATRIXMGR.getMatrix(texMatId);
+	if (texmat != NULL) {
+	  groundTextureMatrix = texmat->getMatrix();
+	}
       }
     }
   }
@@ -326,12 +322,12 @@ void BackgroundRenderer::setupGroundMaterials()
   groundGState[1] = gb.getState();
   gb.reset();
   gb.setTexture(groundTextureID);
-  gb.setTextureMatrix(groundTextureMatrixID);
+  gb.setTextureMatrix(groundTextureMatrix);
   groundGState[2] = gb.getState();
   gb.reset();
   gb.setMaterial(defaultMaterial);
   gb.setTexture(groundTextureID);
-  gb.setTextureMatrix(groundTextureMatrixID);
+  gb.setTextureMatrix(groundTextureMatrix);
   groundGState[3] = gb.getState();
 
 
@@ -554,6 +550,9 @@ void BackgroundRenderer::addCloudDrift(GLfloat uDrift, GLfloat vDrift)
 void BackgroundRenderer::renderSky(SceneRenderer& renderer, bool fullWindow,
 				   bool mirror)
 {
+  if (!BZDBCache::drawSky) {
+    return;
+  }
   if (renderer.useQuality() > 0) {
     drawSky(renderer, mirror);
   } else {
@@ -575,7 +574,7 @@ void BackgroundRenderer::renderSky(SceneRenderer& renderer, bool fullWindow,
     glClear(GL_COLOR_BUFFER_BIT);
 
     // draw ground -- first get the color (assume it's all green)
-    GLfloat groundColor = 0.1f + 0.15f * renderer.getSunColor()[1];
+    GLfloat _groundColor = 0.1f + 0.15f * renderer.getSunColor()[1];
     if (fullWindow && viewType == SceneRenderer::ThreeChannel)
       glScissor(x, y, width, height >> 1);
     else if (fullWindow && viewType == SceneRenderer::Stacked)
@@ -586,8 +585,10 @@ void BackgroundRenderer::renderSky(SceneRenderer& renderer, bool fullWindow,
 #endif
     else
       glScissor(x, y + height - viewHeight, width, (viewHeight + 1) >> 1);
-    if (invert) glClearColor(groundColor, 0.0f, groundColor, 0.0f);
-    else glClearColor(0.0f, groundColor, 0.0f, 0.0f);
+    if (invert)
+      glClearColor(_groundColor, 0.0f, _groundColor, 0.0f);
+    else
+      glClearColor(0.0f, _groundColor, 0.0f, 0.0f);
     glClear(GL_COLOR_BUFFER_BIT);
 
     // back to normal
@@ -621,7 +622,7 @@ void BackgroundRenderer::renderGround(SceneRenderer& renderer,
     glClear(GL_COLOR_BUFFER_BIT);
 
     // draw ground -- first get the color (assume it's all green)
-    GLfloat groundColor = 0.1f + 0.15f * renderer.getSunColor()[1];
+    GLfloat _groundColor = 0.1f + 0.15f * renderer.getSunColor()[1];
     if (fullWindow && viewType == SceneRenderer::ThreeChannel)
       glScissor(x, y, width, height >> 1);
     else if (fullWindow && viewType == SceneRenderer::Stacked)
@@ -632,8 +633,10 @@ void BackgroundRenderer::renderGround(SceneRenderer& renderer,
 #endif
     else
       glScissor(x, y + height - viewHeight, width, (viewHeight + 1) >> 1);
-    if (invert) glClearColor(groundColor, 0.0f, groundColor, 0.0f);
-    else glClearColor(0.0f, groundColor, 0.0f, 0.0f);
+    if (invert)
+      glClearColor(_groundColor, 0.0f, _groundColor, 0.0f);
+    else
+      glClearColor(0.0f, _groundColor, 0.0f, 0.0f);
     glClear(GL_COLOR_BUFFER_BIT);
 
     // back to normal
@@ -664,18 +667,29 @@ void BackgroundRenderer::renderGroundEffects(SceneRenderer& renderer,
     // the ground gets illuminated).  this is necessary because lighting is
     // performed only at a vertex, and the ground's vertices are a few
     // kilometers away.
-    if (BZDBCache::blend && BZDBCache::lighting && !drawingMirror) {
-      drawGroundReceivers(renderer);
+    if (BZDBCache::blend && BZDBCache::lighting &&
+	!drawingMirror && BZDBCache::drawGroundLights) {
+      if (BZDBCache::tesselation && (renderer.useQuality() >= 3)) {
+//	  (BZDB.get(StateDatabase::BZDB_FOGMODE) == "none")) {
+	// not really tesselation, but it is tied to the "Best" lighting,
+	// avoid on foggy maps, because the blending function accumulates
+	// too much brightness.
+	drawAdvancedGroundReceivers(renderer);
+      } else {
+	drawGroundReceivers(renderer);
+      }
     }
 
     if (renderer.useQuality() > 1) {
       // light the mountains (so that they get dark when the sun goes down).
       // don't do zbuffer test since they occlude all drawn before them and
       // are occluded by all drawn after.
-      if (mountainsVisible) drawMountains();
+      if (mountainsVisible && BZDBCache::drawMountains) {
+	drawMountains();
+      }
 
       // draw clouds
-      if (cloudsVisible) {
+      if (cloudsVisible && BZDBCache::drawClouds) {
 	cloudsGState.setState();
 	glMatrixMode(GL_TEXTURE);
 	glPushMatrix();
@@ -695,7 +709,9 @@ void BackgroundRenderer::renderEnvironment(SceneRenderer& renderer, bool update)
   if (!blank) {
     if (update) {
       weather.update();
+      EFFECTS.update();
     }
+    EFFECTS.draw(renderer);
     weather.draw(renderer);
   }
 }
@@ -716,107 +732,300 @@ void BackgroundRenderer::resizeSky() {
 }
 
 
-void BackgroundRenderer::drawSky(SceneRenderer& renderer, bool mirror)
+void BackgroundRenderer::setupSkybox()
 {
-  // rotate sky so that horizon-point-toward-sun-color is actually
-  // toward the sun
-  glPushMatrix();
-  glRotatef((GLfloat)((atan2f(sunDirection[1], sunDirection[0]) * 180.0 + 135.0) / M_PI),
-	    0.0f, 0.0f, 1.0f);
+  haveSkybox = false;
 
-  // draw sky
-  skyGState.setState();
-  if (!doSunset) {
-    // just a pyramid
-    glBegin(GL_TRIANGLE_FAN);
-      glColor3fv(skyZenithColor);
-      glVertex3fv(skyPyramid[4]);
-      glColor3fv(skyCrossSunDirColor);
-      glVertex3fv(skyPyramid[0]);
-      glColor3fv(skySunDirColor);
-      glVertex3fv(skyPyramid[3]);
-      glColor3fv(skyCrossSunDirColor);
-      glVertex3fv(skyPyramid[2]);
-      glColor3fv(skyAntiSunDirColor);
-      glVertex3fv(skyPyramid[1]);
-      glColor3fv(skyCrossSunDirColor);
-      glVertex3fv(skyPyramid[0]);
+  int i;
+  const char *(skyboxNames[6]) = {
+    "LeftSkyboxMaterial",
+    "FrontSkyboxMaterial",
+    "RightSkyboxMaterial",
+    "BackSkyboxMaterial",
+    "TopSkyboxMaterial",
+    "BottomSkyboxMaterial"
+  };
+  TextureManager& tm = TextureManager::instance();
+  const BzMaterial* bzmats[6] = {NULL, NULL, NULL, NULL, NULL, NULL};
+
+  // try to load the textures
+  for (i = 0; i < 6; i++) {
+    bzmats[i] = MATERIALMGR.findMaterial(skyboxNames[i]);
+    if ((bzmats[i] == NULL) || (bzmats[i]->getTextureCount() <= 0)) {
+      break;
+    }
+    skyboxTexID[i] = tm.getTextureID(bzmats[i]->getTextureLocal(0).c_str());
+    if (skyboxTexID[i] < 0) {
+      break;
+    }
+  }
+
+  // unload textures if they were not all successful
+  if (i != 6) {
+    while (i >= 0) {
+      if ((bzmats[i] != NULL) && (bzmats[i]->getTextureCount() > 0)) {
+	// NOTE: this could delete textures the might be used elsewhere
+	tm.removeTexture(bzmats[i]->getTextureLocal(0).c_str());
+      }
+      i--;
+    }
+    return;
+  }
+
+  // reference map specified materials
+  for (i = 0; i < 6; i++) {
+    ((BzMaterial*)bzmats[i])->setReference();
+  }
+
+  // setup the wrap mode
+  skyboxWrapMode = GL_CLAMP;
+#ifdef GL_VERSION_1_2
+  const char* extStr = (const char*) glGetString(GL_EXTENSIONS);
+  if (strstr(extStr, "GL_EXT_texture_edge_clamp") != NULL) {
+    skyboxWrapMode = GL_CLAMP_TO_EDGE;
+  }
+#endif
+
+  // setup the corner colors
+  const int cornerFaces[8][3] = {
+    {5, 0, 1}, {5, 1, 2}, {5, 2, 3}, {5, 3, 0},
+    {4, 0, 1}, {4, 1, 2}, {4, 2, 3}, {4, 3, 0}
+  };
+  for (i = 0; i < 8; i++) {
+    for (int c = 0; c < 4; c++) {
+      skyboxColor[i][c] = 0.0f;
+      for (int f = 0; f < 3; f++) {
+	skyboxColor[i][c] += bzmats[cornerFaces[i][f]]->getDiffuse()[c];
+      }
+      skyboxColor[i][c] /= 3.0f;
+    }
+  }
+
+  haveSkybox = true;
+
+  return;
+}
+
+
+void BackgroundRenderer::drawSkybox()
+{
+  // sky box must fit inside far clipping plane
+  // (adjusted for the deepProjection matrix)
+  const float d = 3.0f * BZDBCache::worldSize;
+  const GLfloat verts[8][3] = {
+    {-d, -d, -d}, {+d, -d, -d}, {+d, +d, -d}, {-d, +d, -d},
+    {-d, -d, +d}, {+d, -d, +d}, {+d, +d, +d}, {-d, +d, +d}
+  };
+  const GLfloat txcds[4][2] = {
+    {1.0f, 0.0f}, {0.0f, 0.0f},
+    {0.0f, 1.0f}, {1.0f, 1.0f}
+  };
+
+  TextureManager& tm = TextureManager::instance();
+
+  OpenGLGState::resetState();
+
+  const GLfloat (*color)[4] = skyboxColor;
+
+  glEnable(GL_TEXTURE_2D);
+  glDisable(GL_CULL_FACE);
+  glShadeModel(GL_SMOOTH);
+
+  if (!BZDBCache::drawGround) {
+    tm.bind(skyboxTexID[5]); // bottom
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, skyboxWrapMode);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, skyboxWrapMode);
+    glBegin(GL_QUADS);
+    {
+      glTexCoord2fv(txcds[0]); glColor3fv(color[2]); glVertex3fv(verts[2]);
+      glTexCoord2fv(txcds[1]); glColor3fv(color[3]); glVertex3fv(verts[3]);
+      glTexCoord2fv(txcds[2]); glColor3fv(color[0]); glVertex3fv(verts[0]);
+      glTexCoord2fv(txcds[3]); glColor3fv(color[1]); glVertex3fv(verts[1]);
+    }
     glEnd();
   }
-  else {
-    // overall shape is a pyramid, but the solar sides are two
-    // triangles each.  the top triangle is all zenith color,
-    // the bottom goes from zenith to sun-dir color.
-    glBegin(GL_TRIANGLE_FAN);
-      glColor3fv(skyZenithColor);
-      glVertex3fv(skyPyramid[4]);
-      glColor3fv(skyCrossSunDirColor);
-      glVertex3fv(skyPyramid[2]);
-      glColor3fv(skyAntiSunDirColor);
-      glVertex3fv(skyPyramid[1]);
-      glColor3fv(skyCrossSunDirColor);
-      glVertex3fv(skyPyramid[0]);
-    glEnd();
 
-    GLfloat sunsetTopPoint[3];
-    sunsetTopPoint[0] = skyPyramid[3][0] * (1.0f - sunsetTop);
-    sunsetTopPoint[1] = skyPyramid[3][1] * (1.0f - sunsetTop);
-    sunsetTopPoint[2] = skyPyramid[4][2] * sunsetTop;
-    glBegin(GL_TRIANGLES);
-      glColor3fv(skyZenithColor);
-      glVertex3fv(skyPyramid[4]);
-      glColor3fv(skyCrossSunDirColor);
-      glVertex3fv(skyPyramid[0]);
-      glColor3fv(skyZenithColor);
-      glVertex3fv(sunsetTopPoint);
-      glVertex3fv(skyPyramid[4]);
-      glVertex3fv(sunsetTopPoint);
-      glColor3fv(skyCrossSunDirColor);
-      glVertex3fv(skyPyramid[2]);
-      glColor3fv(skyZenithColor);
-      glVertex3fv(sunsetTopPoint);
-      glColor3fv(skyCrossSunDirColor);
-      glVertex3fv(skyPyramid[0]);
-      glColor3fv(skySunDirColor);
-      glVertex3fv(skyPyramid[3]);
-      glColor3fv(skyCrossSunDirColor);
-      glVertex3fv(skyPyramid[2]);
-      glColor3fv(skyZenithColor);
-      glVertex3fv(sunsetTopPoint);
-      glColor3fv(skySunDirColor);
-      glVertex3fv(skyPyramid[3]);
-    glEnd();
+  tm.bind(skyboxTexID[4]); // top
+  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, skyboxWrapMode);
+  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, skyboxWrapMode);
+  glBegin(GL_QUADS);
+  {
+    glTexCoord2fv(txcds[0]); glColor3fv(color[5]); glVertex3fv(verts[5]);
+    glTexCoord2fv(txcds[1]); glColor3fv(color[4]); glVertex3fv(verts[4]);
+    glTexCoord2fv(txcds[2]); glColor3fv(color[7]); glVertex3fv(verts[7]);
+    glTexCoord2fv(txcds[3]); glColor3fv(color[6]); glVertex3fv(verts[6]);
+  }
+  glEnd();
+
+  tm.bind(skyboxTexID[0]); // left
+  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, skyboxWrapMode);
+  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, skyboxWrapMode);
+  glBegin(GL_QUADS);
+  {
+    glTexCoord2fv(txcds[0]); glColor3fv(color[0]); glVertex3fv(verts[0]);
+    glTexCoord2fv(txcds[1]); glColor3fv(color[3]); glVertex3fv(verts[3]);
+    glTexCoord2fv(txcds[2]); glColor3fv(color[7]); glVertex3fv(verts[7]);
+    glTexCoord2fv(txcds[3]); glColor3fv(color[4]); glVertex3fv(verts[4]);
+  }
+  glEnd();
+
+  tm.bind(skyboxTexID[1]); // front
+  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, skyboxWrapMode);
+  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, skyboxWrapMode);
+  glBegin(GL_QUADS);
+  {
+    glTexCoord2fv(txcds[0]); glColor3fv(color[1]); glVertex3fv(verts[1]);
+    glTexCoord2fv(txcds[1]); glColor3fv(color[0]); glVertex3fv(verts[0]);
+    glTexCoord2fv(txcds[2]); glColor3fv(color[4]); glVertex3fv(verts[4]);
+    glTexCoord2fv(txcds[3]); glColor3fv(color[5]); glVertex3fv(verts[5]);
+  }
+  glEnd();
+
+  tm.bind(skyboxTexID[2]); // right
+  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, skyboxWrapMode);
+  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, skyboxWrapMode);
+  glBegin(GL_QUADS);
+  {
+    glTexCoord2fv(txcds[0]); glColor3fv(color[2]); glVertex3fv(verts[2]);
+    glTexCoord2fv(txcds[1]); glColor3fv(color[1]); glVertex3fv(verts[1]);
+    glTexCoord2fv(txcds[2]); glColor3fv(color[5]); glVertex3fv(verts[5]);
+    glTexCoord2fv(txcds[3]); glColor3fv(color[6]); glVertex3fv(verts[6]);
+  }
+  glEnd();
+
+  tm.bind(skyboxTexID[3]); // back
+  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, skyboxWrapMode);
+  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, skyboxWrapMode);
+  glBegin(GL_QUADS);
+  {
+    glTexCoord2fv(txcds[0]); glColor3fv(color[3]); glVertex3fv(verts[3]);
+    glTexCoord2fv(txcds[1]); glColor3fv(color[2]); glVertex3fv(verts[2]);
+    glTexCoord2fv(txcds[2]); glColor3fv(color[6]); glVertex3fv(verts[6]);
+    glTexCoord2fv(txcds[3]); glColor3fv(color[7]); glVertex3fv(verts[7]);
+  }
+  glEnd();
+
+  glShadeModel(GL_FLAT);
+  glEnable(GL_CULL_FACE);
+  glDisable(GL_TEXTURE_2D);
+}
+
+
+void BackgroundRenderer::drawSky(SceneRenderer& renderer, bool mirror)
+{
+  glPushMatrix();
+
+  const bool doSkybox = haveSkybox && (renderer.useQuality() >= 2);
+
+  if (!doSkybox) {
+    // rotate sky so that horizon-point-toward-sun-color is actually
+    // toward the sun
+    glRotatef((GLfloat)((atan2f(sunDirection[1], sunDirection[0]) * 180.0 + 135.0) / M_PI),
+	      0.0f, 0.0f, 1.0f);
+
+    // draw sky
+    skyGState.setState();
+    if (!doSunset) {
+      // just a pyramid
+      glBegin(GL_TRIANGLE_FAN);
+	glColor3fv(skyZenithColor);
+	glVertex3fv(skyPyramid[4]);
+	glColor3fv(skyCrossSunDirColor);
+	glVertex3fv(skyPyramid[0]);
+	glColor3fv(skySunDirColor);
+	glVertex3fv(skyPyramid[3]);
+	glColor3fv(skyCrossSunDirColor);
+	glVertex3fv(skyPyramid[2]);
+	glColor3fv(skyAntiSunDirColor);
+	glVertex3fv(skyPyramid[1]);
+	glColor3fv(skyCrossSunDirColor);
+	glVertex3fv(skyPyramid[0]);
+      glEnd();
+    }
+    else {
+      // overall shape is a pyramid, but the solar sides are two
+      // triangles each.  the top triangle is all zenith color,
+      // the bottom goes from zenith to sun-dir color.
+      glBegin(GL_TRIANGLE_FAN);
+	glColor3fv(skyZenithColor);
+	glVertex3fv(skyPyramid[4]);
+	glColor3fv(skyCrossSunDirColor);
+	glVertex3fv(skyPyramid[2]);
+	glColor3fv(skyAntiSunDirColor);
+	glVertex3fv(skyPyramid[1]);
+	glColor3fv(skyCrossSunDirColor);
+	glVertex3fv(skyPyramid[0]);
+      glEnd();
+
+      GLfloat sunsetTopPoint[3];
+      sunsetTopPoint[0] = skyPyramid[3][0] * (1.0f - sunsetTop);
+      sunsetTopPoint[1] = skyPyramid[3][1] * (1.0f - sunsetTop);
+      sunsetTopPoint[2] = skyPyramid[4][2] * sunsetTop;
+      glBegin(GL_TRIANGLES);
+	glColor3fv(skyZenithColor);
+	glVertex3fv(skyPyramid[4]);
+	glColor3fv(skyCrossSunDirColor);
+	glVertex3fv(skyPyramid[0]);
+	glColor3fv(skyZenithColor);
+	glVertex3fv(sunsetTopPoint);
+	glVertex3fv(skyPyramid[4]);
+	glVertex3fv(sunsetTopPoint);
+	glColor3fv(skyCrossSunDirColor);
+	glVertex3fv(skyPyramid[2]);
+	glColor3fv(skyZenithColor);
+	glVertex3fv(sunsetTopPoint);
+	glColor3fv(skyCrossSunDirColor);
+	glVertex3fv(skyPyramid[0]);
+	glColor3fv(skySunDirColor);
+	glVertex3fv(skyPyramid[3]);
+	glColor3fv(skyCrossSunDirColor);
+	glVertex3fv(skyPyramid[2]);
+	glColor3fv(skyZenithColor);
+	glVertex3fv(sunsetTopPoint);
+	glColor3fv(skySunDirColor);
+	glVertex3fv(skyPyramid[3]);
+      glEnd();
+    }
   }
 
   glLoadIdentity();
   renderer.getViewFrustum().executeOrientation();
 
-  if (mirror) {
+  const bool useClipPlane = (mirror && (doSkybox || BZDBCache::drawCelestial));
+
+  if (useClipPlane) {
     glEnable(GL_CLIP_PLANE0);
     const GLdouble plane[4] = {0.0, 0.0, +1.0, 0.0};
     glClipPlane(GL_CLIP_PLANE0, plane);
   }
 
-  if (sunDirection[2] > -0.009f) {
-    sunGState.setState();
-    glColor3fv(renderer.getSunScaledColor());
-    glCallList(sunXFormList);
+  if (doSkybox) {
+    drawSkybox();
   }
 
-  if (doStars) {
-    starGState[starGStateIndex].setState();
-    glCallList(starXFormList);
+  if (BZDBCache::drawCelestial) {
+    if (sunDirection[2] > -0.009f) {
+      sunGState.setState();
+      glColor3fv(renderer.getSunScaledColor());
+      glCallList(sunXFormList);
+    }
+
+    if (doStars) {
+      starGState[starGStateIndex].setState();
+      glCallList(starXFormList);
+    }
+
+    if (moonDirection[2] > -0.009f) {
+      moonGState[doStars ? 1 : 0].setState();
+      glColor3f(1.0f, 1.0f, 1.0f);
+   //   if (useMoonTexture)
+   //     glEnable(GL_TEXTURE_2D);
+      glCallList(moonList);
+    }
+
   }
 
-  if (moonDirection[2] > -0.009f) {
-    moonGState[doStars ? 1 : 0].setState();
-    glColor3f(1.0f, 1.0f, 1.0f);
- //   if (useMoonTexture)
- //     glEnable(GL_TEXTURE_2D);
-    glCallList(moonList);
-  }
-
-  if (mirror) {
+  if (useClipPlane) {
     glDisable(GL_CLIP_PLANE0);
   }
 
@@ -826,27 +1035,28 @@ void BackgroundRenderer::drawSky(SceneRenderer& renderer, bool mirror)
 
 void BackgroundRenderer::drawGround()
 {
-  // draw ground
-  glNormal3f(0.0f, 0.0f, 1.0f);
-  if (invert) {
-    glColor4fv(groundColorInv[styleIndex]);
-    invGroundGState[styleIndex].setState();
-  } else {
-    if (BZDB.isSet("GroundOverideColor")) {
+  if (BZDBCache::drawGround) {
+    // draw ground
+    glNormal3f(0.0f, 0.0f, 1.0f);
+    if (invert) {
+      glColor4fv(groundColorInv[styleIndex]);
+      invGroundGState[styleIndex].setState();
+    } else {
       float color[4];
-      parseColorString(BZDB.get("GroundOverideColor"), color);
-      glColor4fv(color);
+      if (BZDB.isSet("GroundOverideColor") &&
+	  parseColorString(BZDB.get("GroundOverideColor"), color)) {
+	glColor4fv(color);
+      } else {
+	glColor4fv(groundColor[styleIndex]);
+      }
+      groundGState[styleIndex].setState();
     }
-    else {
-      glColor4fv(groundColor[styleIndex]);
-    }
-    groundGState[styleIndex].setState();
-  }
 
-  if (RENDERER.useQuality() >= 3) {
-    drawGroundCentered();
-  } else {
-    glCallList(simpleGroundList[styleIndex]);
+    if (RENDERER.useQuality() >= 3) {
+      drawGroundCentered();
+    } else {
+      glCallList(simpleGroundList[styleIndex]);
+    }
   }
 }
 
@@ -880,17 +1090,19 @@ void BackgroundRenderer::drawGroundCentered()
     {tXmin, tYmin}, {tXmax, tYmin}, {tXmax, tYmax}, {tXmin, tYmax}
   };
 
-  const GLubyte fan[6] = { 0, 1, 2, 3, 4, 1};
+//  const GLubyte fan[6] = { 0, 1, 2, 3, 4, 1};
 
-  glVertexPointer(2, GL_FLOAT, 0, vertices);
-  glEnableClientState(GL_VERTEX_ARRAY);
-  glTexCoordPointer(2, GL_FLOAT, 0, texcoords);
-  glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-
-  glDrawElements(GL_TRIANGLE_FAN, 6, GL_UNSIGNED_BYTE, fan);
-
-  glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-  glDisableClientState(GL_VERTEX_ARRAY);
+  glNormal3f(0.0f, 0.0f, 1.0f);
+  glBegin(GL_TRIANGLE_FAN);
+  {
+    for (int i = 0; i < 5; i++) {
+      glTexCoord2fv(texcoords[i]);
+      glVertex2fv(vertices[i]);
+    }
+    glTexCoord2fv(texcoords[1]);
+    glVertex2fv(vertices[1]);
+  }
+  glEnd();
 
   return;
 }
@@ -950,15 +1162,53 @@ void			BackgroundRenderer::drawGroundShadows(
   // disable color updates
   SceneNode::setColorOverride(true);
 
-  sunShadowsGState.setState();
-  glColor3f(0.0f, 0.0f, 0.0f);
+  // disable the unused arrays
+  glDisableClientState(GL_NORMAL_ARRAY);
+  glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+
+  if (BZDBCache::stencilShadows) {
+    OpenGLGState::resetState();
+    const float shadowAlpha = BZDB.eval("shadowAlpha");
+    glColor4f(0.0f, 0.0f, 0.0f, shadowAlpha);
+    if (shadowAlpha < 1.0f) {
+      // use the stencil to avoid overlapping shadows
+      glClearStencil(0);
+      glClear(GL_STENCIL_BUFFER_BIT);
+      glStencilFunc(GL_NOTEQUAL, 1, 1);
+      glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);
+      glEnable(GL_STENCIL_TEST);
+
+      // turn on blending, and kill culling
+      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+      glEnable(GL_BLEND);
+      glDisable(GL_CULL_FACE);
+    }
+  } else {
+    // use stippling to avoid overlapping shadows
+    sunShadowsGState.setState();
+    glColor3f(0.0f, 0.0f, 0.0f);
+  }
+
+  // render those nodes
   renderer.getShadowList().render();
 
+  // revert to OpenGLGState defaults
+  if (BZDBCache::stencilShadows) {
+    glEnable(GL_CULL_FACE);
+    glDisable(GL_BLEND);
+    glDisable(GL_STENCIL_TEST);
+    glBlendFunc(GL_ONE, GL_ZERO);
+  }
+
   // enable color updates
   SceneNode::setColorOverride(false);
 
   OpenGLGState::resetState();
 
+  // re-enable the arrays
+  glEnableClientState(GL_NORMAL_ARRAY);
+  glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+
   glPopMatrix();
 }
 
@@ -969,8 +1219,8 @@ void BackgroundRenderer::drawGroundReceivers(SceneRenderer& renderer)
   static const int receiverSlices = 8;
   static const float receiverRingSize = 1.2f;	// meters
   static float angle[receiverSlices + 1][2];
-  static bool init = false;
 
+  static bool init = false;
   if (!init) {
     init = true;
     const float receiverSliceAngle = (float)(2.0 * M_PI / double(receiverSlices));
@@ -989,6 +1239,7 @@ void BackgroundRenderer::drawGroundReceivers(SceneRenderer& renderer)
   const float B = 1.0f - (0.6f * renderer.getSunBrightness());
 
   receiverGState.setState();
+
   glPushMatrix();
   int i, j;
   for (int k = 0; k < count; k++) {
@@ -1005,78 +1256,236 @@ void BackgroundRenderer::drawGroundReceivers(SceneRenderer& renderer)
     float d = pos[2];
     float I = B / (atten[0] + d * (atten[1] + d * atten[2]));
 
-    // if I is too low, don't bother drawing anything
-    if (I < 0.02f) {
+    // maximum value
+    const float maxVal = (lightColor[0] > lightColor[1]) ?
+			 ((lightColor[0] > lightColor[2]) ?
+			  lightColor[0] : lightColor[2]) :
+			 ((lightColor[1] > lightColor[2]) ?
+			  lightColor[1] : lightColor[2]);
+
+    // if I is too attenuated, don't bother drawing anything
+    if ((I * maxVal) < 0.02f) {
       continue;
     }
 
     // move to the light's position
     glTranslatef(pos[0], pos[1], 0.0f);
 
-    // modulate light color by ground color
-    float color[3];
-    if (invert) {
-      color[0] = receiverColorInv[0] * lightColor[0];
-      color[1] = receiverColorInv[1] * lightColor[1];
-      color[2] = receiverColorInv[2] * lightColor[2];
-    } else {
-      color[0] = receiverColor[0] * lightColor[0];
-      color[1] = receiverColor[1] * lightColor[1];
-      color[2] = receiverColor[2] * lightColor[2];
-    }
+    // set the main lighting color
+    float color[4];
+    color[0] = lightColor[0];
+    color[1] = lightColor[1];
+    color[2] = lightColor[2];
+    color[3] = I;
 
     // draw ground receiver, computing lighting at each vertex ourselves
     glBegin(GL_TRIANGLE_FAN);
     {
-      glColor3f(I * color[0] > 1.0f ? 1.0f : I * color[0],
-		I * color[1] > 1.0f ? 1.0f : I * color[1],
-		I * color[2] > 1.0f ? 1.0f : I * color[2]);
+      glColor4fv(color);
       glVertex2f(0.0f, 0.0f);
 
       // inner ring
-      d = receiverRingSize + pos[2];
+      d = hypotf(receiverRingSize, pos[2]);
       I = B / (atten[0] + d * (atten[1] + d * atten[2]));
-      I *= pos[2] / hypotf(receiverRingSize, pos[2]);
-      glColor3f(I * color[0] > 1.0f ? 1.0f : I * color[0],
-		I * color[1] > 1.0f ? 1.0f : I * color[1],
-		I * color[2] > 1.0f ? 1.0f : I * color[2]);
+      I *= pos[2] / d;
+      color[3] = I;
+      glColor4fv(color);
       for (j = 0; j <= receiverSlices; j++) {
 	glVertex2f(receiverRingSize * angle[j][0],
 		   receiverRingSize * angle[j][1]);
       }
     }
     glEnd();
+    triangleCount += receiverSlices;
 
     for (i = 1; i < receiverRings; i++) {
       const GLfloat innerSize = receiverRingSize * GLfloat(i * i);
       const GLfloat outerSize = receiverRingSize * GLfloat((i + 1) * (i + 1));
 
       // compute inner and outer lit colors
-      float d = innerSize + pos[2];
-      float I = B / (atten[0] + d * (atten[1] + d * atten[2]));
-      I *= pos[2] / hypotf(innerSize, pos[2]);
-      float innerColor[3];
-      innerColor[0] = I * color[0];
-      innerColor[1] = I * color[1];
-      innerColor[2] = I * color[2];
-      if (innerColor[0] > 1.0f) innerColor[0] = 1.0f;
-      if (innerColor[1] > 1.0f) innerColor[1] = 1.0f;
-      if (innerColor[2] > 1.0f) innerColor[2] = 1.0f;
+      d = hypotf(innerSize, pos[2]);
+      I = B / (atten[0] + d * (atten[1] + d * atten[2]));
+      I *= pos[2] / d;
+      float innerAlpha = I;
 
       if (i + 1 == receiverRings) {
 	I = 0.0f;
       } else {
-	d = outerSize + pos[2];
+	d = hypotf(outerSize, pos[2]);
 	I = B / (atten[0] + d * (atten[1] + d * atten[2]));
-	I *= pos[2] / hypotf(outerSize, pos[2]);
+	I *= pos[2] / d;
       }
-      float outerColor[3];
-      outerColor[0] = I * color[0];
-      outerColor[1] = I * color[1];
-      outerColor[2] = I * color[2];
-      if (outerColor[0] > 1.0f) outerColor[0] = 1.0f;
-      if (outerColor[1] > 1.0f) outerColor[1] = 1.0f;
-      if (outerColor[2] > 1.0f) outerColor[2] = 1.0f;
+      float outerAlpha = I;
+
+      glBegin(GL_QUAD_STRIP);
+      {
+	for (j = 0; j <= receiverSlices; j++) {
+	  color[3] = innerAlpha;
+	  glColor4fv(color);
+	  glVertex2f(angle[j][0] * innerSize, angle[j][1] * innerSize);
+	  color[3] = outerAlpha;
+	  glColor4fv(color);
+	  glVertex2f(angle[j][0] * outerSize, angle[j][1] * outerSize);
+	}
+      }
+      glEnd();
+    }
+    triangleCount += (receiverSlices * receiverRings * 2);
+
+    glTranslatef(-pos[0], -pos[1], 0.0f);
+  }
+  glPopMatrix();
+}
+
+
+void BackgroundRenderer::drawAdvancedGroundReceivers(SceneRenderer& renderer)
+{
+  const float minLuminance = 0.02f;
+  static const int receiverSlices = 32;
+  static const float receiverRingSize = 0.5f;	// meters
+  static float angle[receiverSlices + 1][2];
+
+  static bool init = false;
+  if (!init) {
+    init = true;
+    const float receiverSliceAngle = (float)(2.0 * M_PI / double(receiverSlices));
+    for (int i = 0; i <= receiverSlices; i++) {
+      angle[i][0] = cosf((float)i * receiverSliceAngle);
+      angle[i][1] = sinf((float)i * receiverSliceAngle);
+    }
+  }
+
+  const int count = renderer.getNumAllLights();
+  if (count == 0) {
+    return;
+  }
+
+  // setup the ground tint
+  const GLfloat* gndColor = groundColor[styleIndex];
+  GLfloat overrideColor[4];
+  if (BZDB.isSet("GroundOverideColor") &&
+      parseColorString(BZDB.get("GroundOverideColor"), overrideColor)) {
+    gndColor = overrideColor;
+  }
+
+  const bool useTexture = BZDBCache::texture && (groundTextureID >= 0);
+  OpenGLGState advGState;
+  OpenGLGStateBuilder builder;
+  builder.setShading(GL_SMOOTH);
+  builder.setBlending((GLenum)GL_ONE, (GLenum)GL_ONE);
+  if (useTexture) {
+    builder.setTexture(groundTextureID);
+    builder.setTextureMatrix(groundTextureMatrix);
+  }
+  advGState = builder.getState();
+  advGState.setState();
+
+  // lazy way to get texcoords
+  if (useTexture) {
+    const float repeat = BZDB.eval("groundHighResTexRepeat");
+    const float sPlane[4] = { repeat, 0.0f, 0.0f, 0.0f };
+    const float tPlane[4] = { 0.0f, repeat, 0.0f, 0.0f };
+    glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
+    glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
+    glTexGenfv(GL_S, GL_EYE_PLANE, sPlane);
+    glTexGenfv(GL_T, GL_EYE_PLANE, tPlane);
+    glEnable(GL_TEXTURE_GEN_S);
+    glEnable(GL_TEXTURE_GEN_T);
+  }
+
+  glPushMatrix();
+  int i, j;
+  for (int k = 0; k < count; k++) {
+    const OpenGLLight& light = renderer.getLight(k);
+    if (light.getOnlyReal()) {
+      continue;
+    }
+
+    // get the light parameters
+    const GLfloat* pos = light.getPosition();
+    const GLfloat* lightColor = light.getColor();
+    const GLfloat* atten = light.getAttenuation();
+
+    // point under light
+    float d = pos[2];
+    float I = 1.0f / (atten[0] + d * (atten[1] + d * atten[2]));
+
+    // set the main lighting color
+    float baseColor[3];
+    baseColor[0] = gndColor[0] * lightColor[0];
+    baseColor[1] = gndColor[1] * lightColor[1];
+    baseColor[2] = gndColor[2] * lightColor[2];
+    if (invert) { // beats me, should just color logic op the static nodes
+      baseColor[0] = 1.0f - baseColor[0];
+      baseColor[1] = 1.0f - baseColor[1];
+      baseColor[2] = 1.0f - baseColor[2];
+    }
+
+    // maximum value
+    const float maxVal = (baseColor[0] > baseColor[1]) ?
+			 ((baseColor[0] > baseColor[2]) ?
+			  baseColor[0] : baseColor[2]) :
+			 ((baseColor[1] > baseColor[2]) ?
+			  baseColor[1] : baseColor[2]);
+
+    // if I is too attenuated, don't bother drawing anything
+    if ((I * maxVal) < minLuminance) {
+      continue;
+    }
+
+    // move to the light's position
+    glTranslatef(pos[0], pos[1], 0.0f);
+
+    float innerSize;
+    float innerColor[3];
+    float outerSize;
+    float outerColor[3];
+
+    // draw ground receiver, computing lighting at each vertex ourselves
+    glBegin(GL_TRIANGLE_FAN);
+    {
+      // center point
+      innerColor[0] = I * baseColor[0];
+      innerColor[1] = I * baseColor[1];
+      innerColor[2] = I * baseColor[2];
+      glColor3fv(innerColor);
+      glVertex2f(0.0f, 0.0f);
+
+      // inner ring
+      d = hypotf(receiverRingSize, pos[2]);
+      I = 1.0f / (atten[0] + d * (atten[1] + d * atten[2]));
+      I *= pos[2] / d; // diffuse angle factor
+      outerColor[0] = I * baseColor[0];
+      outerColor[1] = I * baseColor[1];
+      outerColor[2] = I * baseColor[2];
+      glColor3fv(outerColor);
+      outerSize = receiverRingSize;
+      for (j = 0; j <= receiverSlices; j++) {
+	glVertex2f(outerSize * angle[j][0],
+		   outerSize * angle[j][1]);
+      }
+    }
+    glEnd();
+    triangleCount += receiverSlices;
+
+    bool moreRings = true;
+    for (i = 2; moreRings; i++) {
+      // inner ring
+      innerSize = outerSize;
+      memcpy(innerColor, outerColor, sizeof(float[3]));
+
+      // outer ring
+      outerSize = receiverRingSize * GLfloat(i * i);
+      d = hypotf(outerSize, pos[2]);
+      I = 1.0f / (atten[0] + d * (atten[1] + d * atten[2]));
+      I *= pos[2] / d; // diffuse angle factor
+      if ((I * maxVal) < minLuminance) {
+	I = 0.0f;
+	moreRings = false; // bail after this ring
+      }
+      outerColor[0] = I * baseColor[0];
+      outerColor[1] = I * baseColor[1];
+      outerColor[2] = I * baseColor[2];
 
       glBegin(GL_QUAD_STRIP);
       {
@@ -1089,10 +1498,16 @@ void BackgroundRenderer::drawGroundReceivers(SceneRenderer& renderer)
       }
       glEnd();
     }
+    triangleCount += (receiverSlices * 2 * (i - 2));
 
     glTranslatef(-pos[0], -pos[1], 0.0f);
   }
   glPopMatrix();
+
+  if (useTexture) {
+    glDisable(GL_TEXTURE_GEN_S);
+    glDisable(GL_TEXTURE_GEN_T);
+  }
 }
 
 
@@ -1112,6 +1527,7 @@ void BackgroundRenderer::doFreeDisplayLists()
 
   // don't forget the tag-along
   weather.freeContext();
+  EFFECTS.freeContext();
 
   // simpleGroundList[1] && simpleGroundList[3] are copies of [0] & [2]
   simpleGroundList[1] = INVALID_GL_LIST_ID;
@@ -1152,6 +1568,7 @@ void BackgroundRenderer::doInitDisplayLists()
 
   // don't forget the tag-along
   weather.rebuildContext();
+  EFFECTS.rebuildContext();
 
   // need some workarounds on RIVA 128
   bool isRiva128 = (strncmp((const char*)glGetString(GL_RENDERER),
@@ -1253,7 +1670,6 @@ void BackgroundRenderer::doInitDisplayLists()
     }
     glEndList();
   } else {
-    int i, j;
     GLfloat xmin, xmax;
     GLfloat ymin, ymax;
     GLfloat xdist, ydist;
@@ -1458,7 +1874,7 @@ void BackgroundRenderer::doInitDisplayLists()
 	    glTexCoord2f(frac, 0.99f);
 	    glVertex3f(2.25f * worldSize * cosf(angle),
 			 2.25f * worldSize * sinf(angle),
-			 0.45f * worldSize*hightScale);
+			 0.45f * worldSize * hightScale);
 	  }
 	glEnd();
 	glBegin(GL_TRIANGLE_STRIP);
