@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,36 +7,33 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* interface header */
 #include "LocalPlayer.h"
 
-/* system headers */
-#include <string>
-
 /* common implementation headers */
 #include "CommandManager.h"
-#include "WallObstacle.h"
 #include "BZDBCache.h"
 #include "FlagSceneNode.h"
 #include "CollisionManager.h"
 #include "PhysicsDriver.h"
-#include "ObstacleMgr.h"
+#include "BzfEvent.h"
+#include "WallObstacle.h"
+#include "MeshObstacle.h"
 
 /* local implementation headers */
 #include "World.h"
 #include "sound.h"
-#include "RemotePlayer.h"
 #include "ForceFeedback.h"
-
+#include "effectsRenderer.h"
 
 LocalPlayer*		LocalPlayer::mainPlayer = NULL;
 
-LocalPlayer::LocalPlayer(const PlayerId& id,
-			 const char* name, const char* email) :
-  BaseLocalPlayer(id, name, email),
+LocalPlayer::LocalPlayer(const PlayerId& _id,
+			 const char* name, const char* _email) :
+  BaseLocalPlayer(_id, name, _email),
   gettingSound(true),
   server(ServerLink::getServer()),
   location(Dead),
@@ -91,11 +88,6 @@ LocalPlayer::~LocalPlayer()
   delete antidoteFlag;
 }
 
-LocalPlayer*		LocalPlayer::getMyTank()
-{
-  return mainPlayer;
-}
-
 void			LocalPlayer::setMyTank(LocalPlayer* player)
 {
   mainPlayer = player;
@@ -188,7 +180,7 @@ void LocalPlayer::doSlideMotion(float dt, float slideTime,
 
   // BURROW and AGILITY will not be taken into account
   const FlagType* flag = getFlag();
-  float maxSpeed = BZDB.eval(StateDatabase::BZDB_TANKSPEED);
+  float maxSpeed = BZDBCache::tankSpeed;
   if (flag == Flags::Velocity) {
     maxSpeed *= BZDB.eval(StateDatabase::BZDB_VELOCITYAD);
   } else if (flag == Flags::Thief) {
@@ -288,7 +280,7 @@ void			LocalPlayer::doUpdateMotion(float dt)
     } else if (location == Exploding) {
       // see if explosing time has expired
       if (lastTime - getExplodeTime() >= BZDB.eval(StateDatabase::BZDB_EXPLODETIME)) {
-	dt -= (lastTime - getExplodeTime()) - BZDB.eval(StateDatabase::BZDB_EXPLODETIME);
+	dt -= float((lastTime - getExplodeTime()) - BZDB.eval(StateDatabase::BZDB_EXPLODETIME));
 	if (dt < 0.0f) {
 	  dt = 0.0f;
 	}
@@ -365,11 +357,7 @@ void			LocalPlayer::doUpdateMotion(float dt)
 
     // below the ground: however I got there, creep up
     if (oldPosition[2] < groundLimit) {
-#ifdef _MSC_VER
-      newVelocity[2] = max(newVelocity[2], -oldPosition[2] / 2.0f + 0.5f);
-#else
       newVelocity[2] = std::max(newVelocity[2], -oldPosition[2] / 2.0f + 0.5f);
-#endif
     }
   }
 
@@ -454,11 +442,11 @@ void			LocalPlayer::doUpdateMotion(float dt)
       stuckFrameCount = 0;
       // we are using a maximum value on time for frame to avoid lagging problem
       setDesiredSpeed(0.25f);
-      float deltaTime = dt > 0.1f ? 0.1f : dt;
+      float delta = dt > 0.1f ? 0.1f : dt;
       float normalStuck[3];
       obstacle->getNormal(newPos, normalStuck);
       // use all the given speed to exit
-      float movementMax = desiredSpeed * deltaTime;
+      float movementMax = desiredSpeed * delta;
 
       newVelocity[0] = movementMax * normalStuck[0];
       newVelocity[1] = movementMax * normalStuck[1];
@@ -473,7 +461,7 @@ void			LocalPlayer::doUpdateMotion(float dt)
       newPos[1] += newVelocity[1];
       newPos[2] += newVelocity[2];
       // compute time for all other kind of movements
-      timeStep -= deltaTime;
+      timeStep -= delta;
     }
   }
 
@@ -575,11 +563,7 @@ void			LocalPlayer::doUpdateMotion(float dt)
     newPos[1] = tmpPos[1] + (searchTime * newVelocity[1]);
     newPos[2] = tmpPos[2] + (searchTime * newVelocity[2]);
     if (oldPosition[2] < groundLimit) {
-#ifdef _MSC_VER
-      newVelocity[2] = max(newVelocity[2], -oldPosition[2] / 2.0f + 0.5f);
-#else
       newVelocity[2] = std::max(newVelocity[2], -oldPosition[2] / 2.0f + 0.5f);
-#endif
     }
 
 
@@ -697,11 +681,11 @@ void			LocalPlayer::doUpdateMotion(float dt)
     } else {
       // teleport
       const int source = World::getWorld()->getTeleporter(teleporter, face);
-      int target = World::getWorld()->getTeleportTarget(source);
+      int targetTele = World::getWorld()->getTeleportTarget(source);
 
       int outFace;
       const Teleporter* outPort =
-	World::getWorld()->getTeleporter(target, outFace);
+	World::getWorld()->getTeleporter(targetTele, outFace);
       teleporter->getPointWRT(*outPort, face, outFace,
 			      newPos, newVelocity, newAzimuth,
 			      newPos, newVelocity, &newAzimuth);
@@ -717,8 +701,8 @@ void			LocalPlayer::doUpdateMotion(float dt)
 	newAzimuth = oldAzimuth;
       } else {
 	// save teleport info
-	setTeleport(lastTime, source, target);
-	server->sendTeleport(source, target);
+	setTeleport(lastTime, source, targetTele);
+	server->sendTeleport(source, targetTele);
 	if (gettingSound) {
 	  playLocalSound(SFX_TELEPORT);
 	}
@@ -730,11 +714,11 @@ void			LocalPlayer::doUpdateMotion(float dt)
   setPhysicsDriver(-1);
   if ((lastObstacle != NULL) &&
       (lastObstacle->getType() == MeshFace::getClassName())) {
-    const MeshFace* face = (const MeshFace*) lastObstacle;
-    int driverId = face->getPhysicsDriver();
-    const PhysicsDriver* phydrv = PHYDRVMGR.getDriver(driverId);
-    if (phydrv != NULL) {
-      setPhysicsDriver(driverId);
+    const MeshFace* meshFace = (const MeshFace*) lastObstacle;
+    int driverIdent = meshFace->getPhysicsDriver();
+    const PhysicsDriver* phydriver = PHYDRVMGR.getDriver(driverIdent);
+    if (phydriver != NULL) {
+      setPhysicsDriver(driverIdent);
     }
   }
 
@@ -755,18 +739,17 @@ void			LocalPlayer::doUpdateMotion(float dt)
 
   if (justLanded) {
     setLandingSpeed(oldVelocity[2]);
+    EFFECTS.addLandEffect(getColor(),newPos,getAngle());
   }
   if (gettingSound) {
-    const PhysicsDriver* phydrv = PHYDRVMGR.getDriver(getPhysicsDriver());
-    if ((phydrv != NULL) && (phydrv->getLinearVel()[2] > 0.0f)) {
+    const PhysicsDriver* phydriver = PHYDRVMGR.getDriver(getPhysicsDriver());
+    if ((phydriver != NULL) && (phydriver->getLinearVel()[2] > 0.0f)) {
       playLocalSound(SFX_BOUNCE);
       addRemoteSound(PlayerState::BounceSound);
-    }
-    else if (justLanded && !entryDrop) {
+    } else if (justLanded && !entryDrop) {
       playLocalSound(SFX_LAND);
-    }
-    else if ((location == OnGround) &&
-	     (oldPosition[2] == 0.0f) && (newPos[2] < 0.f)) {
+    } else if ((location == OnGround) &&
+	       (oldPosition[2] == 0.0f) && (newPos[2] < 0.f)) {
       playLocalSound(SFX_BURROW);
     }
   }
@@ -809,6 +792,11 @@ void			LocalPlayer::doUpdateMotion(float dt)
     break;
   }
 
+  // burrowed and oscillating tanks get some resistance in their joystick
+  // if they have ff on
+  if ((location == InBuilding) || (newPos[2] < -0.5f))
+    ForceFeedback::solidMatterFriction();
+
   // calculate the list of inside buildings
   insideBuildings.clear();
   if (location == InBuilding) {
@@ -982,27 +970,39 @@ void LocalPlayer::collectInsideBuildings()
 
 float			LocalPlayer::getReloadTime() const
 {
-  float time = jamTime - TimeKeeper::getCurrent();
-  if (time > 0.0f)
-    return time;
-
-    // look for an empty slot
   const int numShots = World::getWorld()->getMaxShots();
+  if (numShots <= 0) {
+    return 0.0f;
+  }
+
+  float time = float(jamTime - TimeKeeper::getCurrent());
+  if (time > 0.0f) {
+    return time;
+  }
+
+  // look for an empty slot
   int i;
-  for (i = 0; i < numShots; i++)
-    if (!shots[i])
+  for (i = 0; i < numShots; i++) {
+    if (!shots[i]) {
       return 0.0f;
+    }
+  }
 
   // look for the shot fired least recently
-  float minTime = shots[0]->getReloadTime() -
-    (shots[0]->getCurrentTime() - shots[0]->getStartTime());
+  float minTime = float(shots[0]->getReloadTime() -
+    (shots[0]->getCurrentTime() - shots[0]->getStartTime()));
   for (i = 1; i < numShots; i++) {
-    const float t = shots[i]->getReloadTime() -
-      (shots[i]->getCurrentTime() - shots[i]->getStartTime());
-    if (t < minTime) minTime = t;
+    const float t = float(shots[i]->getReloadTime() -
+      (shots[i]->getCurrentTime() - shots[i]->getStartTime()));
+    if (t < minTime) {
+      minTime = t;
+    }
+  }
+
+  if (minTime < 0.0f) {
+    minTime = 0.0f;
   }
 
-  if (minTime < 0.0f) minTime = 0.0f;
   return minTime;
 }
 
@@ -1076,9 +1076,9 @@ void			LocalPlayer::restart(const float* pos, float _azimuth)
   setStatus(getStatus() | short(PlayerState::Alive));
 }
 
-void			LocalPlayer::setTeam(TeamColor team)
+void			LocalPlayer::setTeam(TeamColor _team)
 {
-  changeTeam(team);
+  changeTeam(_team);
 }
 
 void			LocalPlayer::setDesiredSpeed(float fracOfMaxSpeed)
@@ -1215,7 +1215,7 @@ bool			LocalPlayer::fireShot()
   // prepare shot
   FiringInfo firingInfo(*this, i + getSalt());
   // FIXME team coloring of shot is never used; it was meant to be used
-  // for rabbit mode to correctly calculcate team kills when rabbit changes
+  // for rabbit mode to correctly calculate team kills when rabbit changes
   firingInfo.shot.team = getTeam();
   if (firingInfo.flagType == Flags::ShockWave) {
     // move shot origin under tank and make it stationary
@@ -1248,14 +1248,18 @@ bool			LocalPlayer::fireShot()
   shots[i] = new LocalShotPath(firingInfo);
 
   // Insert timestamp, useful for dead reckoning jitter fixing
-  const float timeStamp = TimeKeeper::getCurrent() - TimeKeeper::getNullTime();
+  const float timeStamp = float(TimeKeeper::getCurrent() - TimeKeeper::getNullTime());
   firingInfo.timeSent = timeStamp;
 
   server->sendBeginShot(firingInfo);
+
+  if (BZDB.isTrue("enableLocalShotEffect") && SceneRenderer::instance().useQuality() >= 2)
+    EFFECTS.addShotEffect(getColor(), firingInfo.shot.pos, getAngle(), getVelocity());
+
   if (gettingSound) {
     if (firingInfo.flagType == Flags::ShockWave) {
       playLocalSound(SFX_SHOCK);
-      ForceFeedback::shotFired();
+      ForceFeedback::shockwaveFired();
     }
     else if (firingInfo.flagType == Flags::Laser) {
       playLocalSound(SFX_LASER);
@@ -1292,13 +1296,13 @@ void LocalPlayer::forceReload(float time)
 }
 
 
-bool			LocalPlayer::doEndShot(int id, bool isHit, float* pos)
+bool LocalPlayer::doEndShot(int ident, bool isHit, float* pos)
 {
-  const int index = id & 255;
-  const int salt = (id >> 8) & 127;
+  const int index = ident & 255;
+  const int slt   = (ident >> 8) & 127;
 
   // special id used in some messages (and really shouldn't be sent here)
-  if (id == -1)
+  if (ident == -1)
     return false;
 
   // ignore bogus shots (those with a bad index or for shots that don't exist)
@@ -1313,7 +1317,7 @@ bool			LocalPlayer::doEndShot(int id, bool isHit, float* pos)
   // it's possible for an old MsgShotEnd to arrive after we've started a
   // new shot.  that's where the salt comes in.  it changes for each shot
   // so we can identify an old shot from a new one.
-  if (salt != ((shots[index]->getShotId() >> 8) & 127))
+  if (slt != ((shots[index]->getShotId() >> 8) & 127))
     return false;
 
   // keep shot statistics
@@ -1425,7 +1429,7 @@ void			LocalPlayer::explodeTank()
 {
   if (location == Dead || location == Exploding) return;
   float gravity      = BZDBCache::gravity;
-  float explodeTime  = BZDB.eval(StateDatabase::BZDB_EXPLODETIME);
+  float explodeTim   = BZDB.eval(StateDatabase::BZDB_EXPLODETIME);
   // Limiting max height increment to this value (the old default value)
   const float zMax  = 49.0f;
   setExplode(TimeKeeper::getTick());
@@ -1439,7 +1443,7 @@ void			LocalPlayer::explodeTank()
     //   to have a simmetric path (ending at same height as starting)
     //   to reach the acme of parabola, under the max height established
     // take the less
-    newVelocity[2] = - 0.5f * gravity * explodeTime;
+    newVelocity[2] = - 0.5f * gravity * explodeTim;
     maxSpeed       = sqrtf(- 2.0f * zMax * gravity);
     if (newVelocity[2] > maxSpeed)
       newVelocity[2] = maxSpeed;
@@ -1455,9 +1459,9 @@ void			LocalPlayer::doMomentum(float dt,
 						float& speed, float& angVel)
 {
   // get maximum linear and angular accelerations
-  const float linearAcc = (getFlag() == Flags::Momentum) ? BZDB.eval(StateDatabase::BZDB_MOMENTUMLINACC) :
+  float linearAcc = (getFlag() == Flags::Momentum) ? BZDB.eval(StateDatabase::BZDB_MOMENTUMLINACC) :
     World::getWorld()->getLinearAcceleration();
-  const float angularAcc = (getFlag() == Flags::Momentum) ? BZDB.eval(StateDatabase::BZDB_MOMENTUMANGACC) :
+  float angularAcc = (getFlag() == Flags::Momentum) ? BZDB.eval(StateDatabase::BZDB_MOMENTUMANGACC) :
     World::getWorld()->getAngularAcceleration();
 
   // limit linear acceleration
@@ -1470,9 +1474,9 @@ void			LocalPlayer::doMomentum(float dt,
   // limit angular acceleration
   if (angularAcc > 0.0f) {
     const float oldAngVel = getAngularVelocity();
-    const float acc = (angVel - oldAngVel) / dt;
-    if (acc > angularAcc) angVel = oldAngVel + dt * angularAcc;
-    else if (acc < -angularAcc) angVel = oldAngVel - dt * angularAcc;
+    const float angAcc = (angVel - oldAngVel) / dt;
+    if (angAcc > angularAcc) angVel = oldAngVel + dt * angularAcc;
+    else if (angAcc < -angularAcc) angVel = oldAngVel - dt * angularAcc;
   }
 }
 
@@ -1617,8 +1621,8 @@ void			LocalPlayer::changeScore(short deltaWins,
 // calculate overall relative score to other players
 int			 LocalPlayer::getHandicapScoreBase() const
 {
-  int wins = 0;
-  int losses = 0;
+  int winners = 0;
+  int loosers = 0;
   const int maxplayers = World::getWorld()->getMaxPlayers();
 
   // compute overall wins and losses in relation to other players
@@ -1627,10 +1631,10 @@ int			 LocalPlayer::getHandicapScoreBase() const
     if ((player == NULL) || (player->getId() == LocalPlayer::getMyTank()->getId())) {
       continue;
     }
-    wins += player->getLocalWins();
-    losses += player->getLocalLosses();
+    winners += player->getLocalWins();
+    loosers += player->getLocalLosses();
   }
-  return losses - wins;
+  return loosers - winners;
 }
 
 float			LocalPlayer::updateHandicap()
