@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,58 +7,53 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 // interface header
 #include "RadarRenderer.h"
 
-// bzflag common header
-#include "common.h"
-
-// system headers
-#include <stdlib.h>
-#include <string.h>
-#include <vector>
-
 // common implementation headers
-#include "bzfgl.h"
-#include "global.h"
 #include "SceneRenderer.h"
 #include "MainWindow.h"
-#include "Team.h"
-#include "Flag.h"
 #include "OpenGLGState.h"
-#include "StateDatabase.h"
 #include "BZDBCache.h"
 #include "TextureManager.h"
 #include "PhysicsDriver.h"
 #include "ObstacleMgr.h"
-#include "CollisionManager.h"
+#include "MeshSceneNode.h"
+#include "ObstacleList.h"
+#include "WallObstacle.h"
+#include "BoxBuilding.h"
+#include "PyramidBuilding.h"
+#include "MeshObstacle.h"
 
 // local implementation headers
 #include "LocalPlayer.h"
-#include "RemotePlayer.h"
-#include "WorldPlayer.h"
 #include "World.h"
-#include "HUDRenderer.h"
+#include "FlashClock.h"
+#include "ShotPath.h"
 
 
-FlashClock flashTank;
+static FlashClock flashTank;
 static bool toggleTank = false;
 
 const float RadarRenderer::colorFactor = 40.0f;
 
-RadarRenderer::RadarRenderer(const SceneRenderer&, const World& _world)
-						   : world(_world)
+RadarRenderer::RadarRenderer(const SceneRenderer&, World* _world)
+  : world(_world),
+    x(0),
+    y(0),
+    w(0),
+    h(0),
+    dimming(0.0f),
+    decay(0.01f),
+    jammed(false),
+    multiSampled(false)
 {
-  jammed = false;
-  x = y = w = h = 0;
-  decay = 0.01f;
 
   setControlColor();
 
-  multiSampled = false;
 #if defined(GLX_SAMPLES_SGIS) && defined(GLX_SGIS_multisample)
   GLint bits;
   glGetIntergerv(GL_SAMPLES_SGIS, &bits);
@@ -66,6 +61,11 @@ RadarRenderer::RadarRenderer(const SceneRenderer&, const World& _world)
 #endif
 }
 
+void RadarRenderer::setWorld(World* _world)
+{
+  world = _world;
+}
+
 
 void RadarRenderer::setControlColor(const GLfloat *color)
 {
@@ -92,6 +92,12 @@ void RadarRenderer::setJammed(bool _jammed)
 }
 
 
+void RadarRenderer::setDimming(float newDimming)
+{
+  dimming = (1.0f - newDimming > 1.0f) ? 1.0f : (1.0f - newDimming < 0.0f) ? 0.0f : 1.0f - newDimming;
+}
+
+
 void RadarRenderer::drawShot(const ShotPath* shot)
 {
   glBegin(GL_POINTS);
@@ -106,7 +112,7 @@ void RadarRenderer::drawTank(const float pos[3], const Player* player)
 
   // 'ps' is pixel scale, setup in render()
   const float tankRadius = BZDBCache::tankRadius;
-  float minSize = (2.0f * ps);
+  float minSize = 1.5f + (ps * BZDBCache::radarTankPixels);
   GLfloat size;
   if (tankRadius < minSize) {
     size = minSize;
@@ -263,7 +269,15 @@ void RadarRenderer::renderFrame(SceneRenderer& renderer)
   const float top = float(oy + y) - 0.5f;
   const float bottom = float(oy + y + h) + 0.5f;
 
-  glColor3fv(teamColor);
+  float outlineOpacity = RENDERER.getPanelOpacity();
+  float fudgeFactor = BZDBCache::hudGUIBorderOpacityFactor;	// bzdb cache this manybe?
+  if ( outlineOpacity < 1.0f )
+	  outlineOpacity = (outlineOpacity*fudgeFactor) + (1.0f - fudgeFactor);
+
+  if (BZDBCache::blend)
+	  glEnable(GL_BLEND);
+
+  glColor4f(teamColor[0],teamColor[1],teamColor[2],outlineOpacity);
 
   glBegin(GL_LINE_LOOP); {
     glVertex2f(left, top);
@@ -279,6 +293,11 @@ void RadarRenderer::renderFrame(SceneRenderer& renderer)
     glVertex2f(left, bottom);
   } glEnd();
 
+  if (BZDBCache::blend)
+	  glDisable(GL_BLEND);
+
+  glColor4f(teamColor[0],teamColor[1],teamColor[2],1.0f);
+
   const float opacity = renderer.getPanelOpacity();
   if ((opacity < 1.0f) && (opacity > 0.0f)) {
     glScissor(ox + x - 2, oy + y - 2, w + 4, h + 4);
@@ -311,8 +330,11 @@ void RadarRenderer::renderFrame(SceneRenderer& renderer)
 
 void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
 {
+  RenderNode::resetTriangleCount();
+
   const float radarLimit = BZDBCache::radarLimit;
   if (!BZDB.isTrue("displayRadar") || (radarLimit <= 0.0f)) {
+    triangleCount = 0;
     return;
   }
 
@@ -323,20 +345,27 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
     return;
   }
 
+  if (!world) {
+    return;
+  }
+
+  bool rabbitMode = World::getWorld()->allowRabbit();
+
   smooth = !multiSampled && BZDBCache::smooth;
-  const bool fastRadar = ((BZDBCache::radarStyle == 1) && BZDBCache::zbuffer);
+  const bool fastRadar = ((BZDBCache::radarStyle == 1) ||
+			  (BZDBCache::radarStyle == 2)) && BZDBCache::zbuffer;
   const LocalPlayer *myTank = LocalPlayer::getMyTank();
 
   // setup the radar range
-  float range = BZDB.eval("displayRadarRange") * radarLimit;
+  float radarRange = BZDB.eval("displayRadarRange") * radarLimit;
   float maxRange = radarLimit;
   // when burrowed, limit radar range
   if (myTank && (myTank->getFlag() == Flags::Burrow) &&
       (myTank->getPosition()[2] < 0.0f)) {
     maxRange = radarLimit / 4.0f;
   }
-  if (range > maxRange) {
-    range = maxRange;
+  if (radarRange > maxRange) {
+    radarRange = maxRange;
     // only clamp the user's desired range if it's actually
     // greater then 1. otherwise, we may be resetting it due
     // to burrow radar limiting.
@@ -353,9 +382,14 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
   const int ySize = window.getHeight();
   const double xCenter = double(x) + 0.5 * double(w);
   const double yCenter = double(y) + 0.5 * double(h);
-  const double xUnit = 2.0 * range / double(w);
-  const double yUnit = 2.0 * range / double(h);
-  const double maxHeight = (double) COLLISIONMGR.getWorldExtents().maxs[2];
+  const double xUnit = 2.0 * radarRange / double(w);
+  const double yUnit = 2.0 * radarRange / double(h);
+  // NOTE: the visual extents include passable objects
+  double maxHeight = 0.0;
+  const Extents* visExts = renderer.getVisualExtents();
+  if (visExts) {
+    maxHeight = (double)visExts->maxs[2];
+  }
   glOrtho(-xCenter * xUnit, (xSize - xCenter) * xUnit,
 	  -yCenter * yUnit, (ySize - yCenter) * yUnit,
 	  -(maxHeight + 10.0), (maxHeight + 10.0));
@@ -367,6 +401,7 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
 
   OpenGLGState::resetState();
 
+
   // if jammed then draw white noise.  occasionally draw a good frame.
   if (jammed && (bzfrand() > decay)) {
 
@@ -402,13 +437,13 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
       glBegin(GL_QUADS);
       {
 	glTexCoord2f(np[noisePattern+0],np[noisePattern+1]);
-	glVertex2f(-range,-range);
+	glVertex2f(-radarRange,-radarRange);
 	glTexCoord2f(np[noisePattern+2],np[noisePattern+1]);
-	glVertex2f( range,-range);
+	glVertex2f( radarRange,-radarRange);
 	glTexCoord2f(np[noisePattern+2],np[noisePattern+3]);
-	glVertex2f( range, range);
+	glVertex2f( radarRange, radarRange);
 	glTexCoord2f(np[noisePattern+0],np[noisePattern+3]);
-	glVertex2f(-range, range);
+	glVertex2f(-radarRange, radarRange);
       }
       glEnd();
 
@@ -423,22 +458,23 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
       glBegin(GL_QUADS);
       {
 	glTexCoord2f(0,0);
-	glVertex2f(-range,-range);
+	glVertex2f(-radarRange,-radarRange);
 	glTexCoord2f(1,0);
-	glVertex2f( range,-range);
+	glVertex2f( radarRange,-radarRange);
 	glTexCoord2f(1,1);
-	glVertex2f( range, range);
+	glVertex2f( radarRange, radarRange);
 	glTexCoord2f(0,1);
-	glVertex2f(-range, range);
+	glVertex2f(-radarRange, radarRange);
       }
       glEnd();
 
       glDisable(GL_TEXTURE_2D);
     }
     if (decay > 0.015f) decay *= 0.5f;
+
   }
 
-  // only draw if there's a local player
+  // only draw if there's a local player and a world
   else if (myTank) {
     // if decay is sufficiently small then boost it so it's more
     // likely a jammed radar will get a few good frames closely
@@ -449,7 +485,8 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
 
 
     // get size of pixel in model space (assumes radar is square)
-    ps = 2.0f * range / GLfloat(w);
+    ps = 2.0f * (radarRange / GLfloat(w));
+    MeshSceneNode::setRadarLodScale(ps);
 
     float tankWidth = BZDBCache::tankWidth;
     float tankLength = BZDBCache::tankLength;
@@ -460,7 +497,7 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
     } else {
       useTankDimensions = false;
     }
-    if (useTankDimensions && (RENDERER.useQuality() >= 3)) {
+    if (useTankDimensions && (renderer.useQuality() >= 3)) {
       useTankModels = true;
     } else {
       useTankModels = false;
@@ -474,11 +511,11 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
     // view frustum edges
     glColor3f(1.0f, 0.625f, 0.125f);
     const float fovx = renderer.getViewFrustum().getFOVx();
-    const float viewWidth = range * tanf(0.5f * fovx);
+    const float viewWidth = radarRange * tanf(0.5f * fovx);
     glBegin(GL_LINE_STRIP);
-    glVertex2f(-viewWidth, range);
+    glVertex2f(-viewWidth, radarRange);
     glVertex2f(0.0f, 0.0f);
-    glVertex2f(viewWidth, range);
+    glVertex2f(viewWidth, radarRange);
     glEnd();
 
     // transform to the observer's viewpoint
@@ -495,10 +532,8 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
     // setup the blending function
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
-
     // draw the buildings
-    renderObstacles(fastRadar, range);
-
+    renderObstacles(fastRadar, radarRange);
 
     // antialiasing on for lines and points unless we're multisampling,
     // in which case it's automatic and smoothing makes them look worse.
@@ -509,7 +544,7 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
     }
 
     // draw my shots
-    int maxShots = world.getMaxShots();
+    int maxShots = world->getMaxShots();
     int i;
     float muzzleHeight = BZDB.eval(StateDatabase::BZDB_MUZZLEHEIGHT);
     for (i = 0; i < maxShots; i++) {
@@ -533,15 +568,14 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
       }
     }
 
-
     // draw other tanks (and any flags on them)
     // note about flag drawing.  each line segment is drawn twice
     // (once in each direction);  this degrades the antialiasing
     // but on systems that don't do correct filtering of endpoints
     // not doing it makes (half) the endpoints jump wildly.
-    const int curMaxPlayers = world.getCurMaxPlayers();
+    const int curMaxPlayers = world->getCurMaxPlayers();
     for (i = 0; i < curMaxPlayers; i++) {
-      RemotePlayer* player = world.getPlayer(i);
+      RemotePlayer* player = world->getPlayer(i);
       if (!player) {
 	continue;
       }
@@ -563,8 +597,14 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
 
       if (player->isPaused() || player->isNotResponding()) {
 	const float dimfactor = 0.4f;
-	const float *color = Team::getRadarColor(myTank->getFlag() ==
-			     Flags::Colorblindness ? RogueTeam : player->getTeam());
+
+	const float *color;
+	if (myTank->getFlag() == Flags::Colorblindness) {
+	  color = Team::getRadarColor(RogueTeam,rabbitMode);
+	} else {
+	  color = Team::getRadarColor(player->getTeam(),rabbitMode);
+	}
+
 	float dimmedcolor[3];
 	dimmedcolor[0] = color[0] * dimfactor;
 	dimmedcolor[1] = color[1] * dimfactor;
@@ -572,7 +612,7 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
 	glColor3fv(dimmedcolor);
       } else {
 	glColor3fv(Team::getRadarColor(myTank->getFlag() ==
-			     Flags::Colorblindness ? RogueTeam : player->getTeam()));
+			     Flags::Colorblindness ? RogueTeam : player->getTeam(), rabbitMode));
       }
       // If this tank is hunted flash it on the radar
       if (player->isHunted() && myTank->getFlag() != Flags::Colorblindness) {
@@ -601,7 +641,7 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
     bool iSeeAll = myTank && (myTank->getFlag() == Flags::Seer);
     maxShots = World::getWorld()->getMaxShots();
     for (i = 0; i < curMaxPlayers; i++) {
-      RemotePlayer* player = world.getPlayer(i);
+      RemotePlayer* player = world->getPlayer(i);
       if (!player) continue;
       for (int j = 0; j < maxShots; j++) {
 	const ShotPath* shot = player->getShot(j);
@@ -609,9 +649,9 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
 	  const float *shotcolor;
 	  if (coloredShot) {
 	    if (myTank->getFlag() == Flags::Colorblindness)
-	      shotcolor = Team::getRadarColor(RogueTeam);
+	      shotcolor = Team::getRadarColor(RogueTeam,rabbitMode);
 	    else
-	      shotcolor = Team::getRadarColor(player->getTeam());
+	      shotcolor = Team::getRadarColor(player->getTeam(),rabbitMode);
 	    const float cs = colorScale(shot->getPosition()[2], muzzleHeight);
 	    glColor3f(shotcolor[0] * cs, shotcolor[1] * cs, shotcolor[2] * cs);
 	  } else {
@@ -623,10 +663,12 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
     }
 
     // draw flags not on tanks.
-    const int maxFlags = world.getMaxFlags();
+    // draw them in reverse order so that the team flags
+    // (which come first), are drawn on top of the normal flags.
+    const int maxFlags = world->getMaxFlags();
     const bool drawNormalFlags = BZDB.isTrue("displayRadarFlags");
-    for (i = 0; i < maxFlags; i++) {
-      const Flag& flag = world.getFlag(i);
+    for (i = (maxFlags - 1); i >= 0; i--) {
+      const Flag& flag = world->getFlag(i);
       // don't draw flags that don't exist or are on a tank
       if (flag.status == FlagNoExist || flag.status == FlagOnTank)
 	continue;
@@ -651,7 +693,7 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
     glPopMatrix();
 
     // north marker
-    GLfloat ns = 0.05f * range, ny = 0.9f * range;
+    GLfloat ns = 0.05f * radarRange, ny = 0.9f * radarRange;
     glColor3f(1.0f, 1.0f, 1.0f);
     glBegin(GL_LINE_STRIP);
     glVertex2f(-ns, ny - ns);
@@ -665,8 +707,8 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
 
     // forward tick
     glBegin(GL_LINES);
-    glVertex2f(0.0f, range - ps);
-    glVertex2f(0.0f, range - 4.0f * ps);
+    glVertex2f(0.0f, radarRange - ps);
+    glVertex2f(0.0f, radarRange - 4.0f * ps);
     glEnd();
 
     if (!observer) {
@@ -685,15 +727,24 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
       drawTank(myPos, myTank);
     }
 
-    if (smooth) {
-      glDisable(GL_BLEND);
-      glDisable(GL_LINE_SMOOTH);
-      glDisable(GL_POINT_SMOOTH);
+    if (dimming > 0.0f) {
+      if (!smooth) {
+	glEnable(GL_BLEND);
+      }
+      // darken the entire radar if we're dimmed
+      // we're drawing positively, so dimming is actually an opacity
+      glColor4f(0.0f, 0.0f, 0.0f, 1.0f - dimming);
+      glRectf(-radarRange, -radarRange, +radarRange, +radarRange);
     }
+    glDisable(GL_BLEND);
+    glDisable(GL_LINE_SMOOTH);
+    glDisable(GL_POINT_SMOOTH);
   }
 
   // restore GL state
   glPopMatrix();
+
+  triangleCount = RenderNode::getTriangleCount();
 }
 
 
@@ -706,9 +757,9 @@ float RadarRenderer::colorScale(const float z, const float h)
     // Scale color so that objects that are close to tank's level are opaque
     const float zTank = myTank->getPosition()[2];
 
-    if (zTank >= (z + h))
+    if (zTank > (z + h))
       scaleColor = 1.0f - (zTank - (z + h)) / colorFactor;
-    else if (zTank <= z)
+    else if (zTank < z)
       scaleColor = 1.0f - (z - zTank) / colorFactor;
     else
       scaleColor = 1.0f;
@@ -731,9 +782,9 @@ float RadarRenderer::transScale(const float z, const float h)
 
   // Scale color so that objects that are close to tank's level are opaque
   const float zTank = myTank->getPosition()[2];
-  if (zTank >= (z + h))
+  if (zTank > (z + h))
     scaleColor = 1.0f - (zTank - (z + h)) / colorFactor;
-  else if (zTank <= z)
+  else if (zTank < z)
     scaleColor = 1.0f - (z - zTank) / colorFactor;
   else
     scaleColor = 1.0f;
@@ -745,7 +796,7 @@ float RadarRenderer::transScale(const float z, const float h)
 }
 
 
-void RadarRenderer::renderObstacles(bool fastRadar, float range)
+void RadarRenderer::renderObstacles(bool fastRadar, float _range)
 {
   if (smooth) {
     glEnable(GL_BLEND);
@@ -759,7 +810,7 @@ void RadarRenderer::renderObstacles(bool fastRadar, float range)
   if (!fastRadar) {
     renderBoxPyrMesh();
   } else {
-    renderBoxPyrMeshFast(range);
+    renderBoxPyrMeshFast(_range);
   }
 
   // draw the team bases and teleporters
@@ -782,9 +833,9 @@ void RadarRenderer::renderWalls()
   glBegin(GL_LINES);
   for (int i = 0; i < count; i++) {
     const WallObstacle& wall = *((const WallObstacle*) walls[i]);
-    const float w = wall.getBreadth();
-    const float c = w * cosf(wall.getRotation());
-    const float s = w * sinf(wall.getRotation());
+    const float wid = wall.getBreadth();
+    const float c   = wid * cosf(wall.getRotation());
+    const float s   = wid * sinf(wall.getRotation());
     const float* pos = wall.getPosition();
     glVertex2f(pos[0] - s, pos[1] + c);
     glVertex2f(pos[0] + s, pos[1] - c);
@@ -795,7 +846,7 @@ void RadarRenderer::renderWalls()
 }
 
 
-void RadarRenderer::renderBoxPyrMeshFast(float range)
+void RadarRenderer::renderBoxPyrMeshFast(float _range)
 {
   // FIXME - This is hack code at the moment, but even when
   //	 rendering the full world, it draws the aztec map
@@ -834,6 +885,10 @@ void RadarRenderer::renderBoxPyrMeshFast(float range)
   OpenGLGState gs = gb.getState();
   gs.setState();
 
+  // disable the unused arrays
+  glDisableClientState(GL_NORMAL_ARRAY);
+  glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+
   // now that the texture is bound, setup the clamp mode
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
 
@@ -847,8 +902,8 @@ void RadarRenderer::renderBoxPyrMeshFast(float range)
   const float vfz = RENDERER.getViewFrustum().getEye()[2];
   const GLfloat plane[4] =
     { 0.0f, 0.0f, (1.0f / hf), (((hf * 0.5f) - vfz) / hf) };
-  glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
-  glTexGenfv(GL_S, GL_OBJECT_PLANE, plane);
+  glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
+  glTexGenfv(GL_S, GL_EYE_PLANE, plane);
 
   // setup texture generation
   glEnable(GL_TEXTURE_GEN_S);
@@ -856,16 +911,23 @@ void RadarRenderer::renderBoxPyrMeshFast(float range)
   // set the color
   glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
 
-  ViewFrustum radarClipper;
-  // it's also interesting to just use the viewing frustum
-  radarClipper.setOrthoPlanes(RENDERER.getViewFrustum(), range, range);
-  RENDERER.getSceneDatabase()->renderRadarNodes(radarClipper);
+//  if (!BZDB.isTrue("visualRadar")) {
+    ViewFrustum radarClipper;
+    radarClipper.setOrthoPlanes(RENDERER.getViewFrustum(), _range, _range);
+    RENDERER.getSceneDatabase()->renderRadarNodes(radarClipper);
+//  } else {
+//    RENDERER.getSceneDatabase()->renderRadarNodes(RENDERER.getViewFrustum());
+//  }
 
   // restore texture generation
   glDisable(GL_TEXTURE_GEN_S);
 
   OpenGLGState::resetState();
 
+  // re-enable the arrays
+  glEnableClientState(GL_NORMAL_ARRAY);
+  glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+
   // do this after the GState setting
   if (smooth) {
     glEnable(GL_BLEND);
@@ -904,9 +966,9 @@ void RadarRenderer::renderBoxPyrMesh()
     if (box.isInvisible())
       continue;
     const float z = box.getPosition()[2];
-    const float h = box.getHeight();
-    const float cs = colorScale(z, h);
-    glColor4f(0.25f * cs, 0.5f * cs, 0.5f * cs, transScale(z, h));
+    const float bh = box.getHeight();
+    const float cs = colorScale(z, bh);
+    glColor4f(0.25f * cs, 0.5f * cs, 0.5f * cs, transScale(z, bh));
     const float c = cosf(box.getRotation());
     const float s = sinf(box.getRotation());
     const float wx = c * box.getWidth(), wy = s * box.getWidth();
@@ -926,9 +988,9 @@ void RadarRenderer::renderBoxPyrMesh()
   for (i = 0; i < count; i++) {
     const PyramidBuilding& pyr = *((const PyramidBuilding*) pyramids[i]);
     const float z = pyr.getPosition()[2];
-    const float h = pyr.getHeight();
-    const float cs = colorScale(z, h);
-    glColor4f(0.25f * cs, 0.5f * cs, 0.5f * cs, transScale(z, h));
+    const float bh = pyr.getHeight();
+    const float cs = colorScale(z, bh);
+    glColor4f(0.25f * cs, 0.5f * cs, 0.5f * cs, transScale(z, bh));
     const float c = cosf(pyr.getRotation());
     const float s = sinf(pyr.getRotation());
     const float wx = c * pyr.getWidth(), wy = s * pyr.getWidth();
@@ -945,6 +1007,9 @@ void RadarRenderer::renderBoxPyrMesh()
   if (smooth) {
     glEnable(GL_POLYGON_SMOOTH);
   }
+  if (!enhanced) {
+    glDisable(GL_CULL_FACE);
+  }
   const ObstacleList& meshes = OBSTACLEMGR.getMeshes();
   count = meshes.size();
   for (i = 0; i < count; i++) {
@@ -953,18 +1018,24 @@ void RadarRenderer::renderBoxPyrMesh()
 
     for (int f = 0; f < faces; f++) {
       const MeshFace* face = mesh->getFace(f);
-      if (face->getPlane()[2] <= 0.0f) {
-	continue;
+      if (enhanced) {
+	if (face->getPlane()[2] <= 0.0f) {
+	  continue;
+	}
+	const BzMaterial* bzmat = face->getMaterial();
+	if ((bzmat != NULL) && bzmat->getNoRadar()) {
+	  continue;
+	}
       }
       const float z = face->getPosition()[2];
-      const float h = face->getHeight();
-      const float cs = colorScale(z, h);
+      const float bh = face->getHeight();
+      const float cs = colorScale(z, bh);
       // draw death faces with a soupcon of red
       const PhysicsDriver* phydrv = PHYDRVMGR.getDriver(face->getPhysicsDriver());
       if ((phydrv != NULL) && phydrv->getIsDeath()) {
-	glColor4f(0.75f * cs, 0.25f * cs, 0.25f * cs, transScale(z, h));
+	glColor4f(0.75f * cs, 0.25f * cs, 0.25f * cs, transScale(z, bh));
       } else {
-	glColor4f(0.25f * cs, 0.5f * cs, 0.5f * cs, transScale(z, h));
+	glColor4f(0.25f * cs, 0.5f * cs, 0.5f * cs, transScale(z, bh));
       }
       // draw the face as a triangle fan
       int vertexCount = face->getVertexCount();
@@ -976,6 +1047,9 @@ void RadarRenderer::renderBoxPyrMesh()
       glEnd();
     }
   }
+  if (!enhanced) {
+    glEnable(GL_CULL_FACE);
+  }
   if (smooth) {
     glDisable(GL_POLYGON_SMOOTH);
   }
@@ -994,9 +1068,9 @@ void RadarRenderer::renderBoxPyrMesh()
       if (box.isInvisible())
 	continue;
       const float z = box.getPosition()[2];
-      const float h = box.getHeight();
-      const float cs = colorScale(z, h);
-      glColor4f(0.25f * cs, 0.5f * cs, 0.5f * cs, transScale(z, h));
+      const float bh = box.getHeight();
+      const float cs = colorScale(z, bh);
+      glColor4f(0.25f * cs, 0.5f * cs, 0.5f * cs, transScale(z, bh));
       const float c = cosf(box.getRotation());
       const float s = sinf(box.getRotation());
       const float wx = c * box.getWidth(), wy = s * box.getWidth();
@@ -1014,9 +1088,9 @@ void RadarRenderer::renderBoxPyrMesh()
     for (i = 0; i < count; i++) {
       const PyramidBuilding& pyr = *((const PyramidBuilding*) pyramids[i]);
       const float z = pyr.getPosition()[2];
-      const float h = pyr.getHeight();
-      const float cs = colorScale(z, h);
-      glColor4f(0.25f * cs, 0.5f * cs, 0.5f * cs, transScale(z, h));
+      const float bh = pyr.getHeight();
+      const float cs = colorScale(z, bh);
+      glColor4f(0.25f * cs, 0.5f * cs, 0.5f * cs, transScale(z, bh));
       const float c = cosf(pyr.getRotation());
       const float s = sinf(pyr.getRotation());
       const float wx = c * pyr.getWidth(), wy = s * pyr.getWidth();
@@ -1040,10 +1114,10 @@ void RadarRenderer::renderBasesAndTeles()
   int i;
 
   // draw team bases
-  if(world.allowTeamFlags()) {
+  if(world->allowTeamFlags()) {
     for(i = 1; i < NumTeams; i++) {
       for (int j = 0;;j++) {
-	const float *base = world.getBase(i, j);
+	const float *base = world->getBase(i, j);
 	if (base == NULL)
 	  break;
 	glColor3fv(Team::getRadarColor(TeamColor(i)));
@@ -1077,9 +1151,9 @@ void RadarRenderer::renderBasesAndTeles()
     const Teleporter & tele = *((const Teleporter *) teleporters[i]);
     if (tele.isHorizontal ()) {
       const float z = tele.getPosition ()[2];
-      const float h = tele.getHeight ();
-      const float cs = colorScale (z, h);
-      glColor4f (1.0f * cs, 1.0f * cs, 0.25f * cs, transScale (z, h));
+      const float bh = tele.getHeight ();
+      const float cs = colorScale (z, bh);
+      glColor4f (1.0f * cs, 1.0f * cs, 0.25f * cs, transScale (z, bh));
       const float c = cosf (tele.getRotation ());
       const float s = sinf (tele.getRotation ());
       const float wx = c * tele.getWidth (), wy = s * tele.getWidth ();
@@ -1102,12 +1176,12 @@ void RadarRenderer::renderBasesAndTeles()
     }
     else {
       const float z = tele.getPosition ()[2];
-      const float h = tele.getHeight ();
-      const float cs = colorScale (z, h);
-      glColor4f (1.0f * cs, 1.0f * cs, 0.25f * cs, transScale (z, h));
-      const float w = tele.getBreadth ();
-      const float c = w * cosf (tele.getRotation ());
-      const float s = w * sinf (tele.getRotation ());
+      const float bh = tele.getHeight ();
+      const float cs = colorScale (z, bh);
+      glColor4f (1.0f * cs, 1.0f * cs, 0.25f * cs, transScale (z, bh));
+      const float tw = tele.getBreadth ();
+      const float c = tw * cosf (tele.getRotation ());
+      const float s = tw * sinf (tele.getRotation ());
       const float *pos = tele.getPosition ();
       glVertex2f (pos[0] - s, pos[1] + c);
       glVertex2f (pos[0] + s, pos[1] - c);
@@ -1121,6 +1195,12 @@ void RadarRenderer::renderBasesAndTeles()
 }
 
 
+int RadarRenderer::getFrameTriangleCount() const
+{
+  return triangleCount;
+}
+
+
 // Local Variables: ***
 // mode:C++ ***
 // tab-width: 8 ***
