@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* interface header */
@@ -15,28 +15,19 @@
 
 /* system implementation headers */
 #include <assert.h>
-#include <string>
-#include <vector>
 
 /* common implementation headers */
 #include "TextureManager.h"
-#include "BZDBCache.h"
-#include "LaserSceneNode.h"
 #include "Intersect.h"
-#include "Team.h"
-#include "WallObstacle.h"
 #include "BZDBCache.h"
-#include "ObstacleMgr.h"
+#include "WallObstacle.h"
 
 /* local implementation headers */
 #include "sound.h"
 #include "LocalPlayer.h"
 #include "World.h"
-
-/* FIXME -- declarations from player.h */
-Player* lookupPlayer(PlayerId id);
-void addShotExplosion(const float* pos);
-
+#include "effectsRenderer.h"
+#include "playing.h"
 
 SegmentedShotStrategy::SegmentedShotStrategy(ShotPath* _path, bool useSuperTexture, bool faint) :
 				ShotStrategy(_path)
@@ -49,15 +40,24 @@ SegmentedShotStrategy::SegmentedShotStrategy(ShotPath* _path, bool useSuperTextu
   lastSegment = segment = 0;
 
   // get team
-  Player* p = lookupPlayer(_path->getPlayer());
-  team = p ? p->getTeam() : RogueTeam;
+  if (_path->getPlayer() == ServerPlayer) {
+    TeamColor tmpTeam = _path->getFiringInfo().shot.team;
+    team = (tmpTeam < RogueTeam) ? RogueTeam :
+	   (tmpTeam > RabbitTeam) ? RogueTeam : tmpTeam;
+  } else {
+    Player* p = lookupPlayer(_path->getPlayer());
+    team = p ? p->getTeam() : RogueTeam;
+  }
 
   // initialize scene nodes
   boltSceneNode = new BoltSceneNode(_path->getPosition());
-  const float* c = Team::getRadarColor(team);
-  if (faint){
-    boltSceneNode->setColor(c[0], c[1], c[2],0.2f);
-    boltSceneNode->setTextureColor(1.0f,1.0f,1.0f,0.3f);
+
+  bool rabbitMode = World::getWorld()->allowRabbit();
+
+  const float* c = Team::getRadarColor(team, rabbitMode);
+  if (faint) {
+    boltSceneNode->setColor(c[0], c[1], c[2], 0.2f);
+    boltSceneNode->setTextureColor(1.0f, 1.0f, 1.0f, 0.3f);
   } else {
     boltSceneNode->setColor(c[0], c[1], c[2], 1.0f);
   }
@@ -91,16 +91,49 @@ void			SegmentedShotStrategy::update(float dt)
     while (segment < numSegments && segments[segment].end <= currentTime) {
       if (++segment < numSegments) {
 	switch (segments[segment].reason) {
-	  case ShotPathSegment::Ricochet: {
-	    // play ricochet sound.  ricochet of local player's shots
-	    // are important, others are not.
-	    const float* pos = segments[segment].ray.getOrigin();
-	    playWorldSound(SFX_RICOCHET, pos,
-		getPath().getPlayer() == LocalPlayer::getMyTank()->getId());
+	  case ShotPathSegment::Ricochet:
+	    {
+	      // play ricochet sound.  ricochet of local player's shots
+	      // are important, others are not.
+	      const PlayerId myTankId = LocalPlayer::getMyTank()->getId();
+	      const bool important = (getPath().getPlayer() == myTankId);
+	      const float* pos = segments[segment].ray.getOrigin();
+	      playWorldSound(SFX_RICOCHET, pos, important);
+
+	      // this is fugly but it's what we do
+	      float dir[3];
+	      const float* newDir = segments[segment].ray.getDirection();
+	      const float* oldDir = segments[segment - 1].ray.getDirection();
+	      dir[0] = newDir[0] - oldDir[0];
+	      dir[1] = newDir[1] - oldDir[1];
+	      dir[2] = newDir[2] - oldDir[2];
+
+	      float rots[2];
+	      const float horiz = sqrtf((dir[0]*dir[0]) + (dir[1]*dir[1]));
+	      rots[0] = atan2f(dir[1], dir[0]);
+	      rots[1] = atan2f(dir[2], horiz);
+
+	      EFFECTS.addRicoEffect( pos, rots);
+	      break;
+	    }
+	  case ShotPathSegment::Boundary:
 	    break;
-	  }
 	  default:
-	    // currently no sounds for anything else
+	    {
+	      // this is fugly but it's what we do
+	      float dir[3];
+	      dir[0] = segments[segment].ray.getDirection()[0];
+	      dir[1] = segments[segment].ray.getDirection()[1];
+	      dir[2] = segments[segment].ray.getDirection()[2];
+
+	      float rots[2];
+	      const float horiz = sqrtf((dir[0]*dir[0]) + (dir[1]*dir[1]));
+	      rots[0] = atan2f(dir[1], dir[0]);
+	      rots[1] = atan2f(dir[2], horiz);
+
+	      const float* pos = segments[segment].ray.getOrigin();
+	      EFFECTS.addShotTeleportEffect( pos, rots);
+	    }
 	    break;
 	}
       }
@@ -112,20 +145,20 @@ void			SegmentedShotStrategy::update(float dt)
     setExpiring();
 
     if (numSegments > 0) {
-	    ShotPathSegment& segment = segments[numSegments - 1];
-	    const float* dir = segment.ray.getDirection();
-	    const float speed = hypotf(dir[0], hypotf(dir[1], dir[2]));
-	    float pos[3];
-	    segment.ray.getPoint(segment.end - segment.start - 1.0f / speed, pos);
-	    /* NOTE -- comment out to not explode when shot expires */
-	    addShotExplosion(pos);
+      ShotPathSegment &segm = segments[numSegments - 1];
+      const float     *dir  = segm.ray.getDirection();
+      const float speed = hypotf(dir[0], hypotf(dir[1], dir[2]));
+      float pos[3];
+      segm.ray.getPoint(float(segm.end - segm.start - 1.0 / speed), pos);
+      /* NOTE -- comment out to not explode when shot expires */
+      addShotExplosion(pos);
     }
   }
 
   // otherwise update position and velocity
   else {
     float p[3];
-    segments[segment].ray.getPoint(currentTime - segments[segment].start, p);
+    segments[segment].ray.getPoint(float(currentTime - segments[segment].start), p);
     setPosition(p);
     setVelocity(segments[segment].ray.getDirection());
   }
@@ -188,7 +221,7 @@ float			SegmentedShotStrategy::checkHit(const BaseLocalPlayer* tank,
   float shotRadius = BZDB.eval(StateDatabase::BZDB_SHOTRADIUS);
 
   // check each segment in interval (prevTime,currentTime]
-  const float dt = currentTime - prevTime;
+  const float dt = float(currentTime - prevTime);
   const int numSegments = segments.size();
   for (int i = lastSegment; i <= segment && i < numSegments; i++) {
     // can never hit your own first laser segment
@@ -208,7 +241,7 @@ float			SegmentedShotStrategy::checkHit(const BaseLocalPlayer* tank,
 
     // construct relative shot ray:  origin and velocity relative to
     // my tank as a function of time (t=0 is start of the interval).
-    Ray relativeRay(rayMinusRay(s.ray, prevTime - s.start, tankLastMotion, 0.0f));
+    Ray relativeRay(rayMinusRay(s.ray, float(prevTime - s.start), tankLastMotion, 0.0f));
 
     // get hit time
     float t;
@@ -257,15 +290,18 @@ float			SegmentedShotStrategy::checkHit(const BaseLocalPlayer* tank,
 void			SegmentedShotStrategy::addShot(
 				SceneDatabase* scene, bool colorblind)
 {
-  const ShotPath& path = getPath();
-  boltSceneNode->move(path.getPosition(), path.getVelocity());
+  const ShotPath& shotPath = getPath();
+  boltSceneNode->move(shotPath.getPosition(), shotPath.getVelocity());
   if (boltSceneNode->getColorblind() != colorblind) {
     boltSceneNode->setColorblind(colorblind);
-    TeamColor tmpTeam = colorblind ? RogueTeam : team;
-    const float* c = Team::getRadarColor(tmpTeam);
+    TeamColor currentTeam = colorblind ? RogueTeam : team;
+
+    bool rabbitMode = World::getWorld()->allowRabbit();
+    const float* c = Team::getRadarColor(currentTeam, rabbitMode);
     boltSceneNode->setColor(c[0], c[1], c[2]);
+
     TextureManager &tm = TextureManager::instance();
-    std::string imageName = Team::getImagePrefix(team);
+    std::string imageName = Team::getImagePrefix(currentTeam);
     imageName += BZDB.get("boltTexture");
     int texture = tm.getTextureID(imageName.c_str());
     if (texture >= 0)
@@ -331,11 +367,12 @@ void			SegmentedShotStrategy::makeSegments(ObstacleEffect e)
 {
   // compute segments of shot until total length of segments exceeds the
   // lifetime of the shot.
-  const ShotPath& path = getPath();
-  const float* v = path.getVelocity();
-  TimeKeeper startTime = path.getStartTime();
-  float timeLeft = path.getLifetime();
-  float minTime = BZDB.eval(StateDatabase::BZDB_MUZZLEFRONT) / hypotf(v[0], hypotf(v[1], v[2]));
+  const ShotPath &shotPath  = getPath();
+  const float    *v	 = shotPath.getVelocity();
+  TimeKeeper      startTime = shotPath.getStartTime();
+  float timeLeft	    = shotPath.getLifetime();
+  float	   minTime   = BZDB.eval(StateDatabase::BZDB_MUZZLEFRONT)
+    / hypotf(v[0], hypotf(v[1], v[2]));
 
   // if all shots ricochet and obstacle effect is stop, then make it ricochet
   if (e == Stop && World::getWorld()->allShotsRicochet()) {
@@ -347,20 +384,32 @@ void			SegmentedShotStrategy::makeSegments(ObstacleEffect e)
   d[0] = v[0];
   d[1] = v[1];
   d[2] = v[2];		// use v[2] to have jumping affect shot velocity
-  o[0] = path.getPosition()[0];
-  o[1] = path.getPosition()[1];
-  o[2] = path.getPosition()[2];
+  o[0] = shotPath.getPosition()[0];
+  o[1] = shotPath.getPosition()[1];
+  o[2] = shotPath.getPosition()[2];
 
   segments.clear();
   ShotPathSegment::Reason reason = ShotPathSegment::Initial;
   int i;
   const int maxSegment = 100;
+  float worldSize = BZDBCache::worldSize / 2.0f - 0.01f;
   for (i = 0; (i < maxSegment) && (timeLeft > Epsilon); i++) {
     // construct ray and find the first building, teleporter, or outer wall
     float o2[3];
     o2[0] = o[0] - minTime * d[0];
     o2[1] = o[1] - minTime * d[1];
     o2[2] = o[2] - minTime * d[2];
+
+    // Sometime shot start outside world
+    if (o2[0] <= -worldSize)
+      o2[0] = -worldSize;
+    if (o2[0] >= worldSize)
+      o2[0] = worldSize;
+    if (o2[1] <= -worldSize)
+      o2[1] = -worldSize;
+    if (o2[1] >= worldSize)
+      o2[1] = worldSize;
+
     Ray r(o2, d);
     Ray rs(o, d);
     float t = timeLeft + minTime;
@@ -370,13 +419,14 @@ void			SegmentedShotStrategy::makeSegments(ObstacleEffect e)
     const Teleporter* teleporter = getFirstTeleporter(r, Epsilon, t, face);
     t -= minTime;
     minTime = 0.0f;
+    bool ignoreHit = false;
 
     // if hit outer wall with ricochet and hit is above top of wall
     // then ignore hit.
     if (!teleporter && building && (e == Reflect) &&
 	(building->getType() == WallObstacle::getClassName()) &&
 	((o[2] + t * d[2]) > building->getHeight())) {
-      t = timeLeft;
+      ignoreHit = true;
     }
 
     // construct next shot segment and add it to list
@@ -386,8 +436,8 @@ void			SegmentedShotStrategy::makeSegments(ObstacleEffect e)
     } else {
       endTime += t;
     }
-    ShotPathSegment segment(startTime, endTime, rs, reason);
-    segments.push_back(segment);
+    ShotPathSegment segm(startTime, endTime, rs, reason);
+    segments.push_back(segm);
     startTime = endTime;
 
     // used up this much time in segment
@@ -399,9 +449,16 @@ void			SegmentedShotStrategy::makeSegments(ObstacleEffect e)
 
     // check in reverse order to see what we hit first
     reason = ShotPathSegment::Through;
-    if (teleporter) {
+    if (ignoreHit) {
+      // uh...ignore this.  usually used if you shoot over the boundary wall.
+      // just move the point of origin and build the next segment
+      o[0] += t * d[0];
+      o[1] += t * d[1];
+      o[2] += t * d[2];
+      reason = ShotPathSegment::Boundary;
+    } else if (teleporter) {
       // entered teleporter -- teleport it
-      unsigned int seed = path.getShotId() + i;
+      unsigned int seed = shotPath.getShotId() + i;
       int source = World::getWorld()->getTeleporter(teleporter, face);
       int target = World::getWorld()->getTeleportTarget(source, seed);
 
@@ -454,7 +511,7 @@ void			SegmentedShotStrategy::makeSegments(ObstacleEffect e)
 	  o[1] += t * d[1];
 	  o[2] += t * d[2];
 
-	  // reflect direction about normal to building
+	  // reflect direction about normal to ground
 	  float normal[3];
 	  normal[0] = 0.0f;
 	  normal[1] = 0.0f;
@@ -471,24 +528,23 @@ void			SegmentedShotStrategy::makeSegments(ObstacleEffect e)
   // make bounding box for entire path
   const int numSegments = segments.size();
   if (numSegments > 0) {
-    const ShotPathSegment& firstSegment = segments[0];
-    bbox[0][0] = firstSegment.bbox[0][0];
-    bbox[0][1] = firstSegment.bbox[0][1];
-    bbox[0][2] = firstSegment.bbox[0][2];
-    bbox[1][0] = firstSegment.bbox[1][0];
-    bbox[1][1] = firstSegment.bbox[1][1];
-    bbox[1][2] = firstSegment.bbox[1][2];
-    for (int i = 1; i < numSegments; i++) {
-      const ShotPathSegment& segment = segments[i];
-      if (bbox[0][0] > segment.bbox[0][0]) bbox[0][0] = segment.bbox[0][0];
-      if (bbox[1][0] < segment.bbox[1][0]) bbox[1][0] = segment.bbox[1][0];
-      if (bbox[0][1] > segment.bbox[0][1]) bbox[0][1] = segment.bbox[0][1];
-      if (bbox[1][1] < segment.bbox[1][1]) bbox[1][1] = segment.bbox[1][1];
-      if (bbox[0][2] > segment.bbox[0][2]) bbox[0][2] = segment.bbox[0][2];
-      if (bbox[1][2] < segment.bbox[1][2]) bbox[1][2] = segment.bbox[1][2];
+    const ShotPathSegment& firstSeg = segments[0];
+    bbox[0][0] = firstSeg.bbox[0][0];
+    bbox[0][1] = firstSeg.bbox[0][1];
+    bbox[0][2] = firstSeg.bbox[0][2];
+    bbox[1][0] = firstSeg.bbox[1][0];
+    bbox[1][1] = firstSeg.bbox[1][1];
+    bbox[1][2] = firstSeg.bbox[1][2];
+    for (i = 1; i < numSegments; i++) {
+      const ShotPathSegment& segm = segments[i];
+      if (bbox[0][0] > segm.bbox[0][0]) bbox[0][0] = segm.bbox[0][0];
+      if (bbox[1][0] < segm.bbox[1][0]) bbox[1][0] = segm.bbox[1][0];
+      if (bbox[0][1] > segm.bbox[0][1]) bbox[0][1] = segm.bbox[0][1];
+      if (bbox[1][1] < segm.bbox[1][1]) bbox[1][1] = segm.bbox[1][1];
+      if (bbox[0][2] > segm.bbox[0][2]) bbox[0][2] = segm.bbox[0][2];
+      if (bbox[1][2] < segm.bbox[1][2]) bbox[1][2] = segm.bbox[1][2];
     }
-  }
-  else {
+  } else {
     bbox[0][0] = bbox[1][0] = 0.0f;
     bbox[0][1] = bbox[1][1] = 0.0f;
     bbox[0][2] = bbox[1][2] = 0.0f;
@@ -504,8 +560,8 @@ const std::vector<ShotPathSegment>&	SegmentedShotStrategy::getSegments() const
 // NormalShotStrategy
 //
 
-NormalShotStrategy::NormalShotStrategy(ShotPath* path) :
-				SegmentedShotStrategy(path, false)
+NormalShotStrategy::NormalShotStrategy(ShotPath* _path) :
+  SegmentedShotStrategy(_path, false)
 {
   // make segments
   makeSegments(Stop);
@@ -520,17 +576,18 @@ NormalShotStrategy::~NormalShotStrategy()
 // RapidFireStrategy
 //
 
-RapidFireStrategy::RapidFireStrategy(ShotPath* path) :
-				SegmentedShotStrategy(path, false)
+RapidFireStrategy::RapidFireStrategy(ShotPath* _path) :
+  SegmentedShotStrategy(_path, false)
 {
   // speed up shell and decrease lifetime
-  FiringInfo& f = getFiringInfo(path);
+  FiringInfo& f = getFiringInfo(_path);
   f.lifetime *= BZDB.eval(StateDatabase::BZDB_RFIREADLIFE);
   float fireAdVel = BZDB.eval(StateDatabase::BZDB_RFIREADVEL);
   f.shot.vel[0] *= fireAdVel;
   f.shot.vel[1] *= fireAdVel;
   f.shot.vel[2] *= fireAdVel;
-  setReloadTime(path->getReloadTime() / BZDB.eval(StateDatabase::BZDB_RFIREADRATE));
+  setReloadTime(_path->getReloadTime()
+		/ BZDB.eval(StateDatabase::BZDB_RFIREADRATE));
 
   // make segments
   makeSegments(Stop);
@@ -545,18 +602,18 @@ RapidFireStrategy::~RapidFireStrategy()
 // ThiefStrategy
 //
 
-ThiefStrategy::ThiefStrategy(ShotPath* path) :
-				SegmentedShotStrategy(path, false),
-				cumTime(0.0f)
+ThiefStrategy::ThiefStrategy(ShotPath *_path) :
+  SegmentedShotStrategy(_path, false),cumTime(0.0f)
 {
   // speed up shell and decrease lifetime
-  FiringInfo& f = getFiringInfo(path);
+  FiringInfo& f = getFiringInfo(_path);
   f.lifetime *= BZDB.eval(StateDatabase::BZDB_THIEFADLIFE);
   float thiefAdVel = BZDB.eval(StateDatabase::BZDB_THIEFADSHOTVEL);
   f.shot.vel[0] *= thiefAdVel;
   f.shot.vel[1] *= thiefAdVel;
   f.shot.vel[2] *= thiefAdVel;
-  setReloadTime(path->getReloadTime() / BZDB.eval(StateDatabase::BZDB_THIEFADRATE));
+  setReloadTime(_path->getReloadTime()
+		/ BZDB.eval(StateDatabase::BZDB_THIEFADRATE));
 
   // make segments
   makeSegments(Stop);
@@ -571,9 +628,9 @@ ThiefStrategy::ThiefStrategy(ShotPath* path) :
   int texture = tm.getTextureID("thief");
 
   for (int i = 0; i < numSegments; i++) {
-    const ShotPathSegment& segment = getSegments()[i];
-    const float t = segment.end - segment.start;
-    const Ray& ray = segment.ray;
+    const ShotPathSegment& segm = getSegments()[i];
+    const float t = float(segm.end - segm.start);
+    const Ray& ray = segm.ray;
     const float* rawdir = ray.getDirection();
     float dir[3];
     dir[0] = t * rawdir[0];
@@ -611,14 +668,14 @@ void			ThiefStrategy::addShot(SceneDatabase* scene, bool)
 void			ThiefStrategy::radarRender() const
 {
   // draw all segments
-  const std::vector<ShotPathSegment>& segments = getSegments();
-  const int numSegments = segments.size();
+  const std::vector<ShotPathSegment>& segmts = getSegments();
+  const int numSegments = segmts.size();
   glBegin(GL_LINES);
     for (int i = 0; i < numSegments; i++) {
-      const ShotPathSegment& segment = segments[i];
-      const float* origin = segment.ray.getOrigin();
-      const float* direction = segment.ray.getDirection();
-      const float dt = segment.end - segment.start;
+      const ShotPathSegment& segm = segmts[i];
+      const float* origin = segm.ray.getOrigin();
+      const float* direction = segm.ray.getDirection();
+      const float dt = float(segm.end - segm.start);
       glVertex2fv(origin);
       glVertex2f(origin[0] + dt * direction[0], origin[1] + dt * direction[1]);
     }
@@ -635,17 +692,18 @@ bool			ThiefStrategy::isStoppedByHit() const
 // MachineGunStrategy
 //
 
-MachineGunStrategy::MachineGunStrategy(ShotPath* path) :
-				SegmentedShotStrategy(path, false)
+MachineGunStrategy::MachineGunStrategy(ShotPath* _path) :
+  SegmentedShotStrategy(_path, false)
 {
   // speed up shell and decrease lifetime
-  FiringInfo& f = getFiringInfo(path);
+  FiringInfo& f = getFiringInfo(_path);
   f.lifetime *= BZDB.eval(StateDatabase::BZDB_MGUNADLIFE);
   float mgunAdVel = BZDB.eval(StateDatabase::BZDB_MGUNADVEL);
   f.shot.vel[0] *= mgunAdVel;
   f.shot.vel[1] *= mgunAdVel;
   f.shot.vel[2] *= mgunAdVel;
-  setReloadTime(path->getReloadTime() / BZDB.eval(StateDatabase::BZDB_MGUNADRATE));
+  setReloadTime(_path->getReloadTime()
+		/ BZDB.eval(StateDatabase::BZDB_MGUNADRATE));
 
   // make segments
   makeSegments(Stop);
@@ -660,8 +718,8 @@ MachineGunStrategy::~MachineGunStrategy()
 // RicochetStrategy
 //
 
-RicochetStrategy::RicochetStrategy(ShotPath* path) :
-				SegmentedShotStrategy(path, false)
+RicochetStrategy::RicochetStrategy(ShotPath* _path) :
+  SegmentedShotStrategy(_path, false)
 {
   // make segments that bounce
   makeSegments(Reflect);
@@ -676,8 +734,8 @@ RicochetStrategy::~RicochetStrategy()
 // SuperBulletStrategy
 //
 
-SuperBulletStrategy::SuperBulletStrategy(ShotPath* path) :
-				SegmentedShotStrategy(path, true)
+SuperBulletStrategy::SuperBulletStrategy(ShotPath* _path) :
+  SegmentedShotStrategy(_path, true)
 {
   // make segments that go through buildings
   makeSegments(Through);
@@ -689,8 +747,8 @@ SuperBulletStrategy::~SuperBulletStrategy()
 }
 
 
-PhantomBulletStrategy::PhantomBulletStrategy(ShotPath* path) :
-				SegmentedShotStrategy(path, false,true)
+PhantomBulletStrategy::PhantomBulletStrategy(ShotPath* _path) :
+  SegmentedShotStrategy(_path, false,true)
 {
   // make segments that go through buildings
   makeSegments(Through);
@@ -705,18 +763,18 @@ PhantomBulletStrategy::~PhantomBulletStrategy()
 // LaserStrategy
 //
 
-LaserStrategy::LaserStrategy(ShotPath* path) :
-				SegmentedShotStrategy(path, false),
-				cumTime(0.0f)
+LaserStrategy::LaserStrategy(ShotPath* _path) :
+  SegmentedShotStrategy(_path, false), cumTime(0.0f)
 {
   // speed up shell and decrease lifetime
-  FiringInfo& f = getFiringInfo(path);
+  FiringInfo& f = getFiringInfo(_path);
   f.lifetime *= BZDB.eval(StateDatabase::BZDB_LASERADLIFE);
   float laserAdVel = BZDB.eval(StateDatabase::BZDB_LASERADVEL);
   f.shot.vel[0] *= laserAdVel;
   f.shot.vel[1] *= laserAdVel;
   f.shot.vel[2] *= laserAdVel;
-  setReloadTime(path->getReloadTime() / BZDB.eval(StateDatabase::BZDB_LASERADRATE));
+  setReloadTime(_path->getReloadTime()
+		/ BZDB.eval(StateDatabase::BZDB_LASERADRATE));
 
   // make segments
   makeSegments(Stop);
@@ -735,9 +793,9 @@ LaserStrategy::LaserStrategy(ShotPath* path) :
   int texture = tm.getTextureID(imageName.c_str());
 
   for (int i = 0; i < numSegments; i++) {
-    const ShotPathSegment& segment = getSegments()[i];
-    const float t = segment.end - segment.start;
-    const Ray& ray = segment.ray;
+    const ShotPathSegment& segm = getSegments()[i];
+    const float t = float(segm.end - segm.start);
+    const Ray& ray = segm.ray;
     const float* rawdir = ray.getDirection();
     float dir[3];
     dir[0] = t * rawdir[0];
@@ -775,14 +833,14 @@ void			LaserStrategy::addShot(SceneDatabase* scene, bool)
 void			LaserStrategy::radarRender() const
 {
   // draw all segments
-  const std::vector<ShotPathSegment>& segments = getSegments();
-  const int numSegments = segments.size();
+  const std::vector<ShotPathSegment>& segmts = getSegments();
+  const int numSegments = segmts.size();
   glBegin(GL_LINES);
     for (int i = 0; i < numSegments; i++) {
-      const ShotPathSegment& segment = segments[i];
-      const float* origin = segment.ray.getOrigin();
-      const float* direction = segment.ray.getDirection();
-      const float dt = segment.end - segment.start;
+      const ShotPathSegment& segm = segmts[i];
+      const float* origin = segm.ray.getOrigin();
+      const float* direction = segm.ray.getDirection();
+      const float dt = float(segm.end - segm.start);
       glVertex2fv(origin);
       glVertex2f(origin[0] + dt * direction[0], origin[1] + dt * direction[1]);
     }
