@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,16 +7,10 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 
-// BZFlag common header
-#include "common.h"
-
-// System headers
-#include <math.h>
-
 // Interface header
 #include "TrackMarks.h"
 
@@ -27,13 +21,10 @@
 #include "CollisionManager.h"
 #include "PhysicsDriver.h"
 #include "Ray.h"
-#include "bzfgl.h"
 #include "TextureManager.h"
 #include "SceneDatabase.h"
 #include "SceneRenderer.h"
 #include "SceneNode.h"
-#include "OpenGLGState.h"
-#include "OpenGLMaterial.h"
 
 
 using namespace TrackMarks;
@@ -155,7 +146,6 @@ class TrackSceneNode : public SceneNode {
   public:
     TrackSceneNode(const TrackEntry*, TrackType, const OpenGLGState*);
     ~TrackSceneNode();
-    bool isTranslucent() const { return false; }
     void addRenderNodes(SceneRenderer&);
     void update(); // set the sphere properties
 
@@ -187,7 +177,7 @@ static const float TreadMiddle = 0.5f * (TreadOutside + TreadInside);
 static const float TreadMarkWidth = 0.2f;
 
 static OpenGLGState smokeGState;
-static const char smokeTexture[] = "smoke";
+static const char smokeTexture[] = "puddle"; // FIXME - not implemented
 static OpenGLGState puddleGState;
 static const char puddleTexture[] = "puddle";
 static OpenGLGState treadsGState;
@@ -513,6 +503,11 @@ void TrackMarks::update(float dt)
   TrackFadeTime = BZDB.eval(StateDatabase::BZDB_TRACKFADE);
   TrackFadeTime = TrackFadeTime * UserFadeScale;
 
+  if (TrackFadeTime <= 0.0f) {
+    clear();
+    return;
+  }
+
   updateList(SmokeList, dt);
   updateList(PuddleList, dt);
   updateList(TreadsGroundList, dt);
@@ -574,11 +569,6 @@ void TrackMarks::notifyStyleChange()
 
 void TrackMarks::renderGroundTracks()
 {
-  if (TrackFadeTime <= 0.0f) {
-    clear();
-    return;
-  }
-
   TrackEntry* ptr;
 
   // disable the zbuffer for drawing on the ground
@@ -615,11 +605,6 @@ void TrackMarks::renderObstacleTracks()
     return; // this is not for the BSP rendering
   }
 
-  if (TrackFadeTime <= 0.0f) {
-    clear();
-    return;
-  }
-
   TrackEntry* ptr;
 
   // disable the zbuffer writing (these are the last things drawn)
@@ -853,7 +838,7 @@ void TrackRenderNode::render()
     drawSmoke(*te);
   }
   return;
-};
+}
 
 
 //
