@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifndef __CMDLINEOPTIONS_H__
@@ -25,10 +25,13 @@
 #include "Protocol.h"
 #include "Flag.h"
 #include "WordFilter.h"
+#include "TextChunkManager.h"
 
 /* bzfs-specific headers */
 #include "AccessControlList.h"
-#include "TextChunkManager.h"
+
+// avoid dependencies
+class EntryZones;
 
 /* constants provided for general consumption */
 const int MaxPlayers = 200;
@@ -44,6 +47,9 @@ enum RabbitSelection {
 typedef std::map<FlagType*, int> FlagNumberMap;
 typedef std::map<FlagType*,bool> FlagOptionMap;
 
+#define _DEFAULT_LIN_ACCEL 0.0f
+#define _DEFAULT_ANGLE_ACCELL 0.0f
+
 /** CmdLineOptions is a container for any of the bzfs options that may
  * be provided via the command line.
  */
@@ -51,26 +57,27 @@ struct CmdLineOptions
 {
   CmdLineOptions()
   : wksPort(ServerPort), gameStyle(PlainGameStyle),
-    rabbitSelection(ScoreRabbitSelection), msgTimer(0), spamWarnMax(0),
+    rabbitSelection(ScoreRabbitSelection), msgTimer(0), spamWarnMax(5),
     servermsg(""), advertisemsg(""), worldFile(""),
     pingInterface(""), password(""),
     listServerOverridden(false), publicizedTitle(""), publicizedAddress(""),
+    advertiseGroups("EVERYONE"),
     suppressMasterBanList(false), masterBanListOverridden(false),
     maxShots(1), maxTeamScore(0), maxPlayerScore(0),
     numExtraFlags(0), teamKillerKickRatio(0),
     numAllowedFlags(0), shakeWins(0), shakeTimeout(0),
     teamFlagTimeout(30), maxlagwarn(10000), lagwarnthresh(-1.0),
     idlekickthresh(-1.0), timeLimit(0.0f), timeElapsed(0.0f),
-    linearAcceleration(0.0f), angularAcceleration(0.0f), useGivenPort(false),
-    useFallbackPort(false), requireUDP(false), requireIdentify(false), randomBoxes(false),
+    linearAcceleration(_DEFAULT_LIN_ACCEL), angularAcceleration(_DEFAULT_ANGLE_ACCELL), useGivenPort(false),
+    useFallbackPort(false), requireUDP(false), randomBoxes(false),
     randomCTF(false), flagsOnBuildings(false), respawnOnBuildings(false),
     oneGameOnly(false), timeManualStart(false), randomHeights(false),
     useTeleporters(false), teamKillerDies(true), printScore(false),
     publicizeServer(false), replayServer(false), startRecording(false),
+    timestampLog(false), timestampMicros(false),
     filterFilename(""), filterCallsigns(false), filterChat(false), filterSimple(false),
     banTime(300), voteTime(60), vetoTime(2), votesRequired(2),
-    votePercentage(50.1f), voteRepeatTime(300), disableSet(false),
-    disableFlagReset(false), disableBan(false), disableKick(false),
+    votePercentage(50.1f), voteRepeatTime(300),
     autoTeam(false), citySize(5), cacheURL(""), cacheOut("")
   {
     int i;
@@ -107,6 +114,7 @@ struct CmdLineOptions
 
   std::string	publicizedTitle;
   std::string	publicizedAddress;
+  std::string	advertiseGroups;
 
   bool			suppressMasterBanList;
   bool			masterBanListOverridden;
@@ -134,7 +142,6 @@ struct CmdLineOptions
   bool			useGivenPort;
   bool			useFallbackPort;
   bool			requireUDP;
-  bool			requireIdentify;
   bool			randomBoxes;
   bool			randomCTF;
   bool			flagsOnBuildings;
@@ -148,6 +155,9 @@ struct CmdLineOptions
   bool			publicizeServer;
   bool			replayServer;
   bool			startRecording;
+  bool			timestampLog;
+  bool			timestampMicros;
+  bool			countdownPaused;
 
   uint16_t		maxTeam[NumTeams];
   FlagNumberMap		flagCount;
@@ -172,11 +182,6 @@ struct CmdLineOptions
   float votePercentage;
   unsigned short int voteRepeatTime;
 
-  bool disableSet;
-  bool disableFlagReset;
-  bool disableBan;
-  bool disableKick;
-
   std::string		reportFile;
   std::string		reportPipe;
 
@@ -191,11 +196,21 @@ struct CmdLineOptions
 
   std::string	   cacheURL;
   std::string	   cacheOut;
+
+  // plugins
+  typedef struct
+  {
+	  std::string plugin;
+	  std::string command;
+  }pluginDef;
+
+  std::vector<pluginDef>	pluginList;
 };
 
 
 void parse(int argc, char **argv, CmdLineOptions &options, bool fromWorldFile = false);
-
+void finalizeParsing(int argc, char **argv, CmdLineOptions &options, EntryZones& ez);
+bool checkCommaList (const char *list, int maxlen);
 
 #else
 struct CmdLineOptions;
@@ -208,4 +223,3 @@ struct CmdLineOptions;
 // indent-tabs-mode: t ***
 // End: ***
 // ex: shiftwidth=2 tabstop=8
-
