@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifdef _MSC_VER
@@ -35,11 +35,22 @@ PlayerAccessMap	groupAccess;
 PlayerAccessMap	userDatabase;
 PasswordMap	passwordDatabase;
 
+uint8_t GetPlayerProperties(bool registered, bool verified, bool admin) {
+  uint8_t result = 0;
+  if (registered)
+    result |= IsRegistered;
+  if (verified)
+    result |= IsVerified;
+  if (admin)
+    result |= IsAdmin;
+  return result;
+}
+
 void setUserPassword(const std::string &nick, const std::string &pass);
 
 PlayerAccessInfo::PlayerAccessInfo()
   : verified(false), loginTime(TimeKeeper::getCurrent()), loginAttempts (0),
-    Admin(false), passwordAttempts(0)
+    serverop(false), passwordAttempts(0)
 {
   groups.push_back("EVERYONE");
 }
@@ -49,10 +60,6 @@ void PlayerAccessInfo::setName(const char* callSign) {
   makeupper(regName);
 }
 
-bool PlayerAccessInfo::isAccessVerified() const {
-  return verified;
-}
-
 bool PlayerAccessInfo::gotAccessFailure() {
   bool accessFailure = loginAttempts >= 5;
   if (accessFailure)
@@ -67,10 +74,10 @@ void PlayerAccessInfo::setLoginFail() {
 void PlayerAccessInfo::setPermissionRights() {
   verified = true;
   // get their real info
-  PlayerAccessInfo &info = getUserInfo(regName);
-  explicitAllows = info.explicitAllows;
-  explicitDenys = info.explicitDenys;
-  groups = info.groups;
+  PlayerAccessInfo &_info = getUserInfo(regName);
+  explicitAllows = _info.explicitAllows;
+  explicitDenys = _info.explicitDenys;
+  groups = _info.groups;
   DEBUG1("Identify %s\n", regName.c_str());
 }
 
@@ -85,13 +92,24 @@ void PlayerAccessInfo::reloadInfo() {
   }
 }
 
-void PlayerAccessInfo::setAdmin() {
+void PlayerAccessInfo::setOperator() {
   passwordAttempts = 0;
-  Admin = true;
+  serverop = true;
+}
+
+bool PlayerAccessInfo::isOperator() const {
+  return serverop;
 }
 
 bool PlayerAccessInfo::isAdmin() const {
-  return Admin;
+  return isOperator() || hasPerm(ban) || hasPerm(shortBan);
+}
+
+bool PlayerAccessInfo::showAsAdmin() const {
+  if (hasPerm(hideAdmin))
+    return false;
+  else
+    return isAdmin();
 }
 
 bool PlayerAccessInfo::passwordAttemptsMax() {
@@ -125,20 +143,20 @@ bool PlayerAccessInfo::isIdentifyRequired() {
 
 bool PlayerAccessInfo::isAllowedToEnter() {
   return verified || !isRegistered() || !isIdentifyRequired();
-};
+}
 
-bool PlayerAccessInfo::isVerified() {
+bool PlayerAccessInfo::isVerified() const{
   return verified;
-};
+}
 
 void PlayerAccessInfo::storeInfo(const char* pwd) {
-  PlayerAccessInfo info;
-  info.addGroup("VERIFIED");
+  PlayerAccessInfo _info;
+  _info.addGroup("VERIFIED");
 
   if (pwd == NULL) {
     // automatically give global users permission to use local accounts
     // since they either already have it, or there's no existing local account.
-    info.addGroup("LOCAL.GLOBAL");
+    _info.addGroup("LOCAL.GLOBAL");
     setUserPassword(regName.c_str(), "");
     DEBUG1("Global Temp Register %s\n", regName.c_str());
   } else {
@@ -146,7 +164,7 @@ void PlayerAccessInfo::storeInfo(const char* pwd) {
     setUserPassword(regName.c_str(), pass.c_str());
     DEBUG1("Register %s %s\n", regName.c_str(), pwd);
   }
-  userDatabase[regName] = info;
+  userDatabase[regName] = _info;
   updateDatabases();
 }
 
@@ -156,15 +174,7 @@ void PlayerAccessInfo::setPasswd(const std::string&  pwd) {
 }
 
 uint8_t PlayerAccessInfo::getPlayerProperties() {
-  uint8_t result = 0;
-  if (isRegistered())
-    result |= IsRegistered;
-  if (verified)
-    result |= IsVerified;
-  if (!hasPerm(hideAdmin) &&
-      (Admin || hasPerm(ban) || hasPerm(shortBan)))
-    result |= IsAdmin;
-  return result;
+  return GetPlayerProperties(isRegistered(), verified, showAsAdmin());
 }
 
 bool PlayerAccessInfo::exists() {
@@ -174,6 +184,9 @@ bool PlayerAccessInfo::exists() {
 bool PlayerAccessInfo::hasGroup(const std::string &group)
 {
   std::string str = group;
+  if (!str.size())
+    return false;
+
   makeupper(str);
 
   return std::find(groups.begin(), groups.end(), str) != groups.end();
@@ -203,26 +216,70 @@ bool PlayerAccessInfo::removeGroup(const std::string &group)
   return true;
 }
 
-bool PlayerAccessInfo::canSet(const std::string& group) {
+bool PlayerAccessInfo::canSet(const std::string& group)
+{
   if (hasPerm(PlayerAccessInfo::setAll))
     return true;
   return hasGroup(group) && hasPerm(PlayerAccessInfo::setPerms);
 }
 
-bool PlayerAccessInfo::hasPerm(PlayerAccessInfo::AccessPerm right) {
-  if (Admin && (right != hideAdmin))
+bool PlayerAccessInfo::hasPerm(PlayerAccessInfo::AccessPerm right) const
+{
+  if (serverop && (right != hideAdmin))
     return true;
   if (explicitDenys.test(right))
     return false;
   if (explicitAllows.test(right))
     return true;
 
-  for (std::vector<std::string>::iterator itr=groups.begin(); itr!=groups.end(); ++itr) {
+  bool isAllowed = false;
+  for (std::vector<std::string>::const_iterator itr=groups.begin(); itr!=groups.end(); ++itr) {
     PlayerAccessMap::iterator group = groupAccess.find(*itr);
-    if (group != groupAccess.end() && group->second.explicitAllows.test(right))
-      return true;
+    if (group != groupAccess.end()){
+      if (group->second.explicitDenys.test(right))
+      return false;
+      else if (group->second.explicitAllows.test(right))
+	isAllowed = true;
+    }
   }
-  return false;
+  return isAllowed;
+}
+
+// grant and revoke perms used with /mute and /unmute
+void PlayerAccessInfo::grantPerm(PlayerAccessInfo::AccessPerm right)
+{
+  explicitAllows.set(right);
+  explicitDenys.reset(right);
+}
+
+void PlayerAccessInfo::revokePerm(PlayerAccessInfo::AccessPerm right)
+{
+  explicitAllows.reset(right);
+  explicitDenys.set(right);
+}
+
+
+// custom perms are ONLY on groups
+bool	PlayerAccessInfo::hasCustomPerm(const char* right) const
+{
+	if (serverop)
+		return true;
+
+	std::string perm = TextUtils::toupper(std::string(right));
+
+	for (std::vector<std::string>::const_iterator itr=groups.begin(); itr!=groups.end(); ++itr)
+	{
+		PlayerAccessMap::iterator group = groupAccess.find(*itr);
+		if (group != groupAccess.end())
+		{
+			for(unsigned int i = 0; i < group->second.customPerms.size(); i++)
+			{
+				if ( perm == TextUtils::toupper(group->second.customPerms[i]) )
+					return true;
+			}
+		}
+	}
+	return false;
 }
 
 bool userExists(const std::string &nick)
@@ -289,48 +346,64 @@ std::string nameFromPerm(PlayerAccessInfo::AccessPerm perm)
 {
   switch (perm) {
     case PlayerAccessInfo::actionMessage: return "actionMessage";
-    case PlayerAccessInfo::adminMessageSend: return "adminMessageSend";
     case PlayerAccessInfo::adminMessageReceive: return "adminMessageReceive";
+    case PlayerAccessInfo::adminMessageSend: return "adminMessageSend";
     case PlayerAccessInfo::antiban : return "antiban";
     case PlayerAccessInfo::antideregister : return "antideregister";
     case PlayerAccessInfo::antikick : return "antikick";
+    case PlayerAccessInfo::antikill : return "antikill";
     case PlayerAccessInfo::antipoll : return "antipoll";
     case PlayerAccessInfo::antipollban : return "antipollban";
     case PlayerAccessInfo::antipollkick : return "antipollkick";
+    case PlayerAccessInfo::antipollkill : return "antipollkill";
     case PlayerAccessInfo::ban: return "ban";
     case PlayerAccessInfo::banlist: return "banlist";
     case PlayerAccessInfo::countdown: return "countdown";
+    case PlayerAccessInfo::date: return "date";
     case PlayerAccessInfo::endGame: return "endGame";
     case PlayerAccessInfo::flagHistory: return "flagHistory";
+    case PlayerAccessInfo::flagMaster: return "flagMaster";
     case PlayerAccessInfo::flagMod: return "flagMod";
     case PlayerAccessInfo::hideAdmin: return "hideAdmin";
     case PlayerAccessInfo::idleStats: return "idleStats";
     case PlayerAccessInfo::info: return "info";
     case PlayerAccessInfo::kick: return "kick";
+    case PlayerAccessInfo::kill: return "kill";
     case PlayerAccessInfo::lagStats: return "lagStats";
     case PlayerAccessInfo::lagwarn: return "lagwarn";
     case PlayerAccessInfo::listPerms: return "listPerms";
     case PlayerAccessInfo::masterBan: return "masterban";
+    case PlayerAccessInfo::mute: return "mute";
     case PlayerAccessInfo::playerList: return "playerList";
     case PlayerAccessInfo::poll: return "poll";
+    case PlayerAccessInfo::pollBan: return "pollBan";
+    case PlayerAccessInfo::pollKick: return "pollKick";
+    case PlayerAccessInfo::pollKill: return "pollKill";
+    case PlayerAccessInfo::pollSet: return "pollSet";
+    case PlayerAccessInfo::pollFlagReset: return "pollFlagReset";
     case PlayerAccessInfo::privateMessage: return "privateMessage";
     case PlayerAccessInfo::record: return "record";
+    case PlayerAccessInfo::rejoin: return "rejoin";
     case PlayerAccessInfo::removePerms: return "removePerms";
     case PlayerAccessInfo::replay: return "replay";
     case PlayerAccessInfo::requireIdentify: return "requireIdentify";
+    case PlayerAccessInfo::say: return "say";
+    case PlayerAccessInfo::sendHelp : return "sendHelp";
     case PlayerAccessInfo::setAll: return "setAll";
     case PlayerAccessInfo::setPassword: return "setPassword";
     case PlayerAccessInfo::setPerms: return "setPerms";
     case PlayerAccessInfo::setVar: return "setVar";
     case PlayerAccessInfo::showOthers: return "showOthers";
+    case PlayerAccessInfo::shortBan: return "shortBan";
     case PlayerAccessInfo::shutdownServer: return "shutdownServer";
+    case PlayerAccessInfo::spawn: return "spawn";
     case PlayerAccessInfo::superKill: return "superKill";
+    case PlayerAccessInfo::talk: return "talk";
     case PlayerAccessInfo::unban: return "unban";
+    case PlayerAccessInfo::unmute: return "unmute";
     case PlayerAccessInfo::veto: return "veto";
     case PlayerAccessInfo::viewReports: return "viewReports";
-    case PlayerAccessInfo::date: return "date";
     case PlayerAccessInfo::vote: return "vote";
-    case PlayerAccessInfo::shortBan: return "shortBan";
     default: return TextUtils::format("UNKNOWN_PERMISSION: %d", (int)perm).c_str();
   };
 }
@@ -338,36 +411,49 @@ std::string nameFromPerm(PlayerAccessInfo::AccessPerm perm)
 PlayerAccessInfo::AccessPerm permFromName(const std::string &name)
 {
   if (name == "ACTIONMESSAGE") return PlayerAccessInfo::actionMessage;
-  if (name == "ADMINMESSAGESEND") return PlayerAccessInfo::adminMessageSend;
   if (name == "ADMINMESSAGERECEIVE") return PlayerAccessInfo::adminMessageReceive;
+  if (name == "ADMINMESSAGESEND") return PlayerAccessInfo::adminMessageSend;
   if (name == "ANTIBAN") return PlayerAccessInfo::antiban;
   if (name == "ANTIDEREGISTER") return PlayerAccessInfo::antideregister;
   if (name == "ANTIKICK") return PlayerAccessInfo::antikick;
+  if (name == "ANTIKILL") return PlayerAccessInfo::antikill;
   if (name == "ANTIPOLL") return PlayerAccessInfo::antipoll;
   if (name == "ANTIPOLLBAN") return PlayerAccessInfo::antipollban;
   if (name == "ANTIPOLLKICK") return PlayerAccessInfo::antipollkick;
+  if (name == "ANTIPOLLKILL") return PlayerAccessInfo::antipollkill;
   if (name == "BAN") return PlayerAccessInfo::ban;
   if (name == "BANLIST") return PlayerAccessInfo::banlist;
   if (name == "COUNTDOWN") return PlayerAccessInfo::countdown;
+  if (name == "DATE") return PlayerAccessInfo::date;
   if (name == "ENDGAME") return PlayerAccessInfo::endGame;
   if (name == "FLAGHISTORY") return PlayerAccessInfo::flagHistory;
+  if (name == "FLAGMASTER") return PlayerAccessInfo::flagMaster;
   if (name == "FLAGMOD") return PlayerAccessInfo::flagMod;
   if (name == "HIDEADMIN") return PlayerAccessInfo::hideAdmin;
   if (name == "IDLESTATS") return PlayerAccessInfo::idleStats;
   if (name == "INFO") return PlayerAccessInfo::info;
   if (name == "KICK") return PlayerAccessInfo::kick;
-  if (name == "DATE") return PlayerAccessInfo::date;
+  if (name == "KILL") return PlayerAccessInfo::kill;
   if (name == "LAGSTATS") return PlayerAccessInfo::lagStats;
   if (name == "LAGWARN") return PlayerAccessInfo::lagwarn;
   if (name == "LISTPERMS") return PlayerAccessInfo::listPerms;
   if (name == "MASTERBAN") return PlayerAccessInfo::masterBan;
+  if (name == "MUTE") return PlayerAccessInfo::mute;
   if (name == "PLAYERLIST") return PlayerAccessInfo::playerList;
   if (name == "POLL") return PlayerAccessInfo::poll;
+  if (name == "POLLBAN") return PlayerAccessInfo::pollBan;
+  if (name == "POLLKICK") return PlayerAccessInfo::pollKick;
+  if (name == "POLLKILL") return PlayerAccessInfo::pollKill;
+  if (name == "POLLSET") return PlayerAccessInfo::pollSet;
+  if (name == "POLLFLAGRESET") return PlayerAccessInfo::pollFlagReset;
   if (name == "PRIVATEMESSAGE") return PlayerAccessInfo::privateMessage;
   if (name == "RECORD") return PlayerAccessInfo::record;
+  if (name == "REJOIN") return PlayerAccessInfo::rejoin;
   if (name == "REMOVEPERMS") return PlayerAccessInfo::removePerms;
   if (name == "REPLAY") return PlayerAccessInfo::replay;
   if (name == "REQUIREIDENTIFY") return PlayerAccessInfo::requireIdentify;
+  if (name == "SAY") return PlayerAccessInfo::say;
+  if (name == "SENDHELP") return PlayerAccessInfo::sendHelp;
   if (name == "SETALL") return PlayerAccessInfo::setAll;
   if (name == "SETPASSWORD") return PlayerAccessInfo::setPassword;
   if (name == "SETPERMS") return PlayerAccessInfo::setPerms;
@@ -375,31 +461,118 @@ PlayerAccessInfo::AccessPerm permFromName(const std::string &name)
   if (name == "SHORTBAN") return PlayerAccessInfo::shortBan;
   if (name == "SHOWOTHERS") return PlayerAccessInfo::showOthers;
   if (name == "SHUTDOWNSERVER") return PlayerAccessInfo::shutdownServer;
+  if (name == "SPAWN") return PlayerAccessInfo::spawn;
   if (name == "SUPERKILL") return PlayerAccessInfo::superKill;
+  if (name == "TALK") return PlayerAccessInfo::talk;
   if (name == "UNBAN") return PlayerAccessInfo::unban;
+  if (name == "UNMUTE") return PlayerAccessInfo::unmute;
   if (name == "VETO") return PlayerAccessInfo::veto;
   if (name == "VIEWREPORTS") return PlayerAccessInfo::viewReports;
   if (name == "VOTE") return PlayerAccessInfo::vote;
   return PlayerAccessInfo::lastPerm;
 }
 
-void parsePermissionString(const std::string &permissionString, std::bitset<PlayerAccessInfo::lastPerm> &perms)
+void parsePermissionString(const std::string &permissionString, PlayerAccessInfo &info)
 {
   if (permissionString.length() < 1)
     return;
-  perms.reset();
 
   std::istringstream permStream(permissionString);
   std::string word;
 
   while (permStream >> word) {
     makeupper(word);
+
+    // do we have an operator? check for a leading, non alphabetic character
+    char first = NULL;
+    if (!TextUtils::isAlphabetic(word[0])) {
+      first = word[0];
+      word.erase(0,1);
+    }
+
+    // Operators are not allowed for userdb
+    if (!info.groupState.test(PlayerAccessInfo::isGroup) && first != NULL){
+      DEBUG1("userdb: illegal permission string, operators are not allowed in userdb\n");
+      return;
+    }
+
+    // if we have an operator, lets handle it
+    // operators are only allowed for groups, they make no sense for userdb
+    if (info.groupState.test(PlayerAccessInfo::isGroup) && first != NULL) {
+      switch (first) {
+	case '*': {
+	  // referenced group
+	  // don't copy setThis, groups have to be named explicitly for every group
+	  // to prevent unexpected side effects
+	  PlayerAccessMap::iterator refgroup = groupAccess.find(word);
+	  if (refgroup != groupAccess.end()) {
+	    info.explicitAllows |= refgroup->second.explicitAllows;
+	    info.explicitDenys |= refgroup->second.explicitDenys;
+	    refgroup->second.groupState.set(PlayerAccessInfo::isReferenced);
+	  } else {
+	    DEBUG1("groupdb: unknown group \"%s\" was referenced\n", word.c_str());
+	  }
+
+	  continue;
+	}
+
+      case '!': {
+	// forbid a permission
+	PlayerAccessInfo::AccessPerm perm = permFromName(word);
+	if (perm != PlayerAccessInfo::lastPerm) {
+	  info.explicitDenys.set(perm);
+	} else {
+          if (word == "ALL") {
+            info.explicitDenys.set();
+            info.explicitDenys[PlayerAccessInfo::lastPerm] = false;
+          } else {
+            DEBUG1("groupdb: Cannot forbid unknown permission %s\n", word.c_str());
+          }
+        }
+
+	continue;
+      }
+
+      case '-': {
+	// remove a permission
+	PlayerAccessInfo::AccessPerm perm = permFromName(word);
+	if (perm != PlayerAccessInfo::lastPerm) {
+	  info.explicitAllows.reset(perm);
+	} else {
+	  if (word == "ALL")
+	    info.explicitAllows.reset();
+	  else
+	    DEBUG1("groupdb: Cannot remove unknown permission %s\n", word.c_str());
+	}
+
+	continue;
+      }
+
+      // + is like no operator, just let it pass trough
+      case '+': break;
+
+      default:
+	DEBUG1("groupdb: ignoring unknown operator type %c\n", first);
+      }
+    }
+
+    // regular permission
     PlayerAccessInfo::AccessPerm perm = permFromName(word);
-    if (perm != PlayerAccessInfo::lastPerm)
-      perms.set(perm);
+    if (perm != PlayerAccessInfo::lastPerm) {
+      info.explicitAllows.set(perm);
+    } else {
+      if (word == "ALL") {
+	info.explicitAllows.set();
+	info.explicitAllows[PlayerAccessInfo::lastPerm] = false;
+      } else {
+	//DEBUG1("groupdb: Cannot set unknown permission %s\n", word.c_str());
+				info.customPerms.push_back(word);
+      }
+    }
   }
 }
 
+
 bool readPassFile(const std::string &filename)
 {
   std::ifstream in(filename.c_str());
@@ -445,18 +618,52 @@ bool PlayerAccessInfo::readGroupsFile(const std::string &filename)
   if (!in)
     return false;
 
+  int linenum = 0;
   std::string line;
   while (std::getline(in, line)) {
+    linenum++;
+
+    // check for a comment string
+    bool skip = true;
+    for (std::string::size_type pos = 0; pos < line.size(); pos++) {
+      const char c = line[pos];
+      if (!TextUtils::isWhitespace(c)) {
+	if (c != '#') {
+	  skip = false;
+	}
+	break;
+      }
+    }
+    if (skip) continue;
+
+    makeupper(line);
+
     std::string::size_type colonpos = line.find(':');
     if (colonpos != std::string::npos) {
       std::string name = line.substr(0, colonpos);
       std::string perm = line.substr(colonpos + 1);
-      makeupper(name);
+
+      // check if we already have this group, else make a new
       PlayerAccessInfo info;
-      parsePermissionString(perm, info.explicitAllows);
+      PlayerAccessMap::iterator oldgroup = groupAccess.find(name);
+      if (oldgroup != groupAccess.end())
+	info = oldgroup->second;
+      else
+	info.groupState[PlayerAccessInfo::isGroup] = true;
+
+      // don't allow changing permissions for a group
+      // that was used as a reference before
+      if (info.groupState.test(isReferenced)) {
+	DEBUG1("groupdb: skipping groupdb line (%i), group was used as reference before\n", linenum);
+	continue;
+      }
+      parsePermissionString(perm, info);
       info.verified = true;
       groupAccess[name] = info;
+    } else {
+      DEBUG1("WARNING: bad groupdb line (%i)\n", linenum);
     }
+
   }
 
   return true;
@@ -488,10 +695,14 @@ bool PlayerAccessInfo::readPermsFile(const std::string &filename)
     // 3rd line - allows
     std::string perms;
     std::getline(in, perms);
-    parsePermissionString(perms, info.explicitAllows);
+    parsePermissionString(perms, info);
 
+    // 4th line - denies
+    // FIXME: not nice ... any ideas how to make it better?
     std::getline(in, perms);
-    parsePermissionString(perms, info.explicitDenys);
+    PlayerAccessInfo dummy;
+    parsePermissionString(perms, dummy);
+    info.explicitDenys = dummy.explicitAllows;
 
     userDatabase[name] = info;
   }
@@ -537,9 +748,9 @@ std::string		userDatabaseFile;
 
 void PlayerAccessInfo::updateDatabases()
 {
-  if(passFile.size())
+  if (passFile.size())
     writePassFile(passFile);
-  if(userDatabaseFile.size())
+  if (userDatabaseFile.size())
     writePermsFile(userDatabaseFile);
 }
 
