@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2004 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifndef BZF_WAVE_AUDIO_FILE_H
@@ -15,14 +15,17 @@
 
 #include "AudioFile.h"
 
+/** This class represents a WAVE file. It implements the read() function from
+    AudioFile. */
 class WaveAudioFile : public AudioFile {
 public:
   WaveAudioFile(std::istream*);
   virtual ~WaveAudioFile();
 
+  /** This function returns the default extension of WAVE audio files. */
   static std::string	getExtension();
 
-  // AudioFile overrides
+  /** This function reads data from a WAVE file. */
   virtual bool		read(void* buffer, int numFrames);
 
 protected:
