@@ -1,6 +1,6 @@
 
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -8,7 +8,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include "common.h"
@@ -29,7 +29,7 @@ static const int defaultAudioRate=22050;
 // SDLMedia
 //
 
-SDLMedia::SDLMedia() : BzfMedia()
+SDLMedia::SDLMedia() : BzfMedia(), outputBuffer(NULL)
 {
   cmdFill       = 0;
   audioReady    = false;
@@ -190,7 +190,7 @@ void SDLMedia::fillAudioWrapper (void * userdata, Uint8 * stream, int len)
 {
   SDLMedia * me = (SDLMedia *) userdata;
   me->fillAudio(stream, len);
-};
+}
 
 void			SDLMedia::writeAudioFrames(
 				const float* samples, int numFrames)
@@ -233,7 +233,7 @@ void	SDLMedia::setDriver(std::string driverName) {
   strncpy(envAssign, envVar.c_str(), 255);
   envAssign[255]     = '\0';
   putenv(envAssign);
-};
+}
 
 // Setting Audio Device
 void	SDLMedia::setDevice(std::string deviceName) {
@@ -242,7 +242,7 @@ void	SDLMedia::setDevice(std::string deviceName) {
   strncpy(envAssign, envVar.c_str(), 255);
   envAssign[255]     = '\0';
   putenv(envAssign);
-};
+}
 
 float*	    SDLMedia::doReadSound(const std::string &filename, int &numFrames,
 				  int &rate) const
