@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2006 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* XDisplay:
@@ -32,6 +32,7 @@ class XDisplay : public BzfDisplay {
     bool		isValid() const;
     bool		isEventPending() const;
     bool		getEvent(BzfEvent&) const;
+    bool		peekEvent(BzfEvent&) const;
 
     // for other X stuff
     class Rep {
@@ -58,6 +59,7 @@ class XDisplay : public BzfDisplay {
 			XDisplay(const XDisplay&);
     XDisplay&		operator=(const XDisplay&);
 
+    bool		setupEvent(BzfEvent&, const XEvent&) const;
     bool		getKey(const XEvent&, BzfKeyEvent&) const;
 
     bool		doSetResolution(int);
